package com.gct.aoid.aop;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.cache.interceptor.SimpleKeyGenerator;
import org.springframework.stereotype.Component;

import java.util.concurrent.ConcurrentHashMap;

@Aspect
@Component
@Slf4j
public class FeignClientAspect {


    private static final String FEIGN_CLIENT_CACHE_PREFIX = "FEIGN-CLIENT-CACHE:";

    public static final ConcurrentHashMap<String,Object> cacheMap = new ConcurrentHashMap<>();

    /**
     * 缓存rpc调用的环绕通知
     * @param joinPoint
     * @return 目标方法调用的结果
     * @throws Throwable
     */
    @Around("@annotation(com.gct.aoid.aop.FeignCache)")
    public Object cacheAround(ProceedingJoinPoint joinPoint) throws Throwable {
        String key = FEIGN_CLIENT_CACHE_PREFIX + joinPoint.getSignature().getDeclaringTypeName() + "#" + joinPoint.getSignature().getName() + "#"
                + SimpleKeyGenerator.generateKey(joinPoint.getArgs());
        Object cache = cacheMap.get(key);
        if (cache != null) {
            return cache;
        }
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        FeignCache annotation = signature.getMethod().getAnnotation(FeignCache.class);
        // 这里可以对结果进行过滤，缓存正确的结果，视具体的业务逻辑来定
        cache = joinPoint.proceed(joinPoint.getArgs());
        if (cacheMap.size()>5000)cacheMap.clear();
        cacheMap.put(key, cache);
        return cache;
    }

}
