package com.gct.aoid.controller;


import com.alibaba.fastjson.JSONObject;
import com.gct.aoid.service.YieldErrorAnalysisService;
import com.gct.aoid.service.impl.YieldErrorAnalysisCalculationService;
import com.gct.common.core.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gxt
 * @since 2021-01-28
 */
@Api(value = "YieldErrorAnalysisController", description = "功图量液")
@RestController
@RequestMapping("/yieldErrorAnalysis")
@Slf4j
public class YieldErrorAnalysisController {
    @Autowired
    private YieldErrorAnalysisService yieldErrorAnalysisService;

    @Autowired
    private YieldErrorAnalysisCalculationService yieldErrorAnalysisCalculationService;
    public static AtomicInteger atomicIntegerYieldError = new AtomicInteger(0);



    /**
     * 分页查询误差率详细井号
     * @param pageNum
     * @param pageSize
     * @param zyq
     * @param wcStart
     * @param wcEnd
     * @param cylStart
     * @param cylEnd
     * @param dateStart
     * @param dateEnd
     * @return
     */


    @ApiOperation(value = "分页查询", notes = "分页查询误差率详细井号")
    @GetMapping("/findWellWcByPages")
    public Result findWellWcByPages(
            @ApiParam(name = "pageNum", value = "当前页", required = true) @RequestParam("pageNum") Integer pageNum,
            @ApiParam(name = "pageSize", value = "页大小", required = true) @RequestParam("pageSize") Integer pageSize,
            @ApiParam(name = "zyq", value = "作业区") @RequestParam(value = "zyq", required = false) String zyq,
            @ApiParam(name = "wcStart", value = "误差开始") @RequestParam(value = "wcStart", required = false) String wcStart,
            @ApiParam(name = "wcEnd", value = "误差结束") @RequestParam(value = "wcEnd", required = false) String wcEnd,
            @ApiParam(name = "cylStart", value = "产液量开始") @RequestParam(value = "cylStart", required = false) String cylStart,
            @ApiParam(name = "cylEnd", value = "产液量结束") @RequestParam(value = "cylEnd", required = false) String cylEnd,
            @ApiParam(name = "jh", value = "井号") @RequestParam(value = "jh", required = false) String jh,
            @ApiParam(name = "dateStart", value = "日期开始") @RequestParam(value = "dateStart", required = false) String dateStart,
            @ApiParam(name = "dateEnd", value = "日期结束") @RequestParam(value = "dateEnd", required = false) String dateEnd
    ) {
        return Result.ok(yieldErrorAnalysisService.findWellWc(pageNum, pageSize, zyq, wcStart, wcEnd, cylStart,cylEnd,jh,dateStart,dateEnd));
    }


    @ApiOperation(value="误差率分析导出")
    @PostMapping("/exportExcelYieldError")
    public Result exportExcelYieldError(@RequestBody JSONObject param){

        String jh =  param.getString("jh");
        String zyq = param.getString("zyq");
        String dateStart = param.getString("dateStart");
        String dateEnd = param.getString("dateEnd");
        String wcStart = param.getString("wcStart");
        String wcEnd = param.getString("wcEnd");
        String cylStart = param.getString("cylStart");
        String cylEnd = param.getString("cylEnd");
        JSONObject jo = yieldErrorAnalysisService.exportExcelYieldError(zyq, wcStart, wcEnd, cylStart,cylEnd,jh,dateStart,dateEnd);
        return jo.getJSONArray("data").size() == 0 ? Result.error("无数据") : Result.ok(jo);
    }




    @ApiOperation(value = "findparameterYieldError(根据参数进行误差率分析计算)", notes = "根据参数进行误差率分析计算")
    @GetMapping("/findparameterYieldError")
    public void findparameterYieldError(
            @ApiParam(name = "dateStart",value = "时间") @RequestParam("dateStart") String dateStart,
            @ApiParam(name = "dateEnd",value = "时间") @RequestParam("dateEnd") String dateEnd

    ) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {

        if (atomicIntegerYieldError.get()!=0) {
            return;
        }

        ExecutorService pool = Executors.newSingleThreadExecutor();
        pool.execute(new Runnable() {
            @SneakyThrows
            @Override
            public void run() {
                atomicIntegerYieldError.set(2);
                yieldErrorAnalysisCalculationService.calYieldError(dateStart,dateEnd);  // 0 没有计算任务 1 计算完成  2 计算中
                atomicIntegerYieldError.set(1);
            }
        });

    }
    @ApiOperation(value = "getParamCalTaskExecutorYieldError(查看重新计算的进度)", notes = "查看重新计算的进度")
    @GetMapping("/getParamCalTaskExecutorYieldError")
    public Result getParamCalTaskExecutorYieldError (){
        int i = atomicIntegerYieldError.get();
        if (i==1){
            atomicIntegerYieldError.set(0);
        }
        return Result.ok(i);
    }



}

