package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidLiquidAbnormalDynamic;
import com.gct.aoid.entity.AoidLiquidThreshold;
import com.gct.aoid.entity.Vo.AbnormalDynamicVo;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.mapper.AoidLiquidAbnormalDynamicMapper;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidLiquidAbnormalDynamicService;
import com.gct.aoid.service.AoidLiquidThresholdService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 液量异常信息表(动态存储) 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-08-15
 */
@Service
public class AoidLiquidAbnormalDynamicServiceImpl extends ServiceImpl<AoidLiquidAbnormalDynamicMapper, AoidLiquidAbnormalDynamic> implements AoidLiquidAbnormalDynamicService {

    @Autowired
    AoidDailyYieldService aoidDailyYieldService;

    @Autowired
    AoidLiquidThresholdService aoidLiquidThresholdService;

    @Autowired
    BaseDataClient baseDataClient;

    @Override
    public void saveLiquidAbnormalList(LocalDateTime date) {
        List<AoidLiquidAbnormalDynamic> abnormalDynamics = new ArrayList<>();
        Map<String, List<AoidLiquidThreshold>> thresholdMap = aoidLiquidThresholdService.getBaseMapper().selectList(null)
                .stream().collect(Collectors.groupingBy(AoidLiquidThreshold::getAppertain));
        for (Map.Entry<String, List<AoidLiquidThreshold>> threshold : thresholdMap.entrySet()) {
            List<String> wellIds = getWellIds(threshold.getKey());
            List<String> list = JSONObject.parseObject(threshold.getValue().get(0).getRequirement(), List.class);
            Map<String, List<AoidDailyYield>> collect = getAoidYields(wellIds, date, list).stream().collect(Collectors.groupingBy(AoidDailyYield::getWellId));
            for (Map.Entry<String, List<AoidDailyYield>> entry : collect.entrySet()) {
                AoidLiquidAbnormalDynamic abnormalDynamic = saveLiquidAbnormal(date, threshold.getValue(), entry.getValue(), list);
                Map<String, Object> map = JSONObject.parseObject(abnormalDynamic.getLiquidInfo(), Map.class);
                if (!StringUtil.isBlankOrEmpty(abnormalDynamic.getWellId()) && map.entrySet().size() > 0)
                    abnormalDynamics.add(abnormalDynamic);
            }
        }
        setAreaData(abnormalDynamics);
        saveBatch(abnormalDynamics);
    }

    @Override
    public Page<AoidLiquidAbnormalDynamic> getAbnormalDynamicList(AbnormalDynamicVo abnormalDynamicVo) {
        Page<AoidLiquidAbnormalDynamic> page = baseMapper.selectPage(new Page<>(abnormalDynamicVo.getCurrentPage(), abnormalDynamicVo.getPageSize()),
                getWrapper(abnormalDynamicVo));
        if (!StringUtil.isBlankOrEmpty(abnormalDynamicVo.getColumnFilter()) && StringUtil.isBlankOrEmpty(abnormalDynamicVo.getHeadFilter())) {
            // 字段过滤
            List<AoidLiquidAbnormalDynamic> columnfilter = getColumn(abnormalDynamicVo);
            page.setTotal(columnfilter.size())
                    .setCurrent(abnormalDynamicVo.getCurrentPage())
                    .setSize(abnormalDynamicVo.getPageSize())
                    .setRecords(page(columnfilter, abnormalDynamicVo));
            return page;
        } else if (StringUtil.isBlankOrEmpty(abnormalDynamicVo.getColumnFilter()) && !StringUtil.isBlankOrEmpty(abnormalDynamicVo.getHeadFilter())) {
            //头部统一过滤
            List<AoidLiquidAbnormalDynamic> abnormalDynamics = headFilter(abnormalDynamicVo);
            page.setTotal(abnormalDynamics.size())
                    .setCurrent(abnormalDynamicVo.getCurrentPage())
                    .setSize(abnormalDynamicVo.getPageSize())
                    .setRecords(page(abnormalDynamics, abnormalDynamicVo));
            return page;
        } else return page;
    }

    private List<AoidLiquidAbnormalDynamic> page(List<AoidLiquidAbnormalDynamic> data, AbnormalDynamicVo abnormalDynamicVo) {
        return data.stream().skip((long) (abnormalDynamicVo.getCurrentPage() - 1) * abnormalDynamicVo.getPageSize()).limit(abnormalDynamicVo.getPageSize()).collect(Collectors.toList());
    }

    private List<AoidLiquidAbnormalDynamic> headFilter(AbnormalDynamicVo abnormalDynamicVo) {
        String headFilter = abnormalDynamicVo.getHeadFilter();
        List<AoidLiquidAbnormalDynamic> data = baseMapper.selectList(getWrapper(abnormalDynamicVo));
        if ("increase".equals(headFilter)) {
            data.forEach(x -> {
                Map<String, Object> map = JSONObject.parseObject(x.getLiquidInfo(), Map.class);
                map.values().removeIf(z -> {
                    Map<String, BigDecimal> doubleMap = JSONObject.parseObject(JSON.toJSONString(z), Map.class);
                    return doubleMap.get("liqDifference").doubleValue() < 0;
                });
                x.setLiquidInfo(JSON.toJSONString(map));
            });
        }
        if ("decrease".equals(headFilter)) {
            data.forEach(x -> {
                Map<String, Object> map = JSONObject.parseObject(x.getLiquidInfo(), Map.class);
                map.values().removeIf(z -> {
                    Map<String, BigDecimal> doubleMap = JSONObject.parseObject(JSON.toJSONString(z), Map.class);
                    return doubleMap.get("liqDifference").doubleValue() > 0;
                });
                x.setLiquidInfo(JSON.toJSONString(map));
            });
        }
        data.removeIf(x -> {
            Map<String, Object> map = JSONObject.parseObject(x.getLiquidInfo(), Map.class);
            return map.keySet().size() == 0 ;
        });
        return data;
    }


    private List<AoidLiquidAbnormalDynamic> getColumn(AbnormalDynamicVo abnormalDynamicVo) {
        List<AoidLiquidAbnormalDynamic> abnormalDynamics = baseMapper.selectList(getWrapper(abnormalDynamicVo));
        List<AoidLiquidAbnormalDynamic> resList = new ArrayList<>();
        String[] columnFilter = abnormalDynamicVo.getColumnFilter().split(",");
        String labelName = columnFilter[0].substring(columnFilter[0].indexOf("-") + 1);
        int filter = Integer.parseInt(columnFilter[1]);
        if (filter < 0) {
            resList.addAll(columnFilter(abnormalDynamics, filter, labelName));
        } else {
            resList.addAll(columnFilter(abnormalDynamics, filter, labelName));
        }
        return resList;
    }

    private List<AoidLiquidAbnormalDynamic> columnFilter(List<AoidLiquidAbnormalDynamic> data, int filter, String labelName) {
        List<AoidLiquidAbnormalDynamic> reduceList = new ArrayList<>();
        List<AoidLiquidAbnormalDynamic> increaseList = new ArrayList<>();
        data.forEach(x -> {
            Map<String, Object> map = JSONObject.parseObject(x.getLiquidInfo(), Map.class);
            if (map.containsKey(labelName)) {
                Map<String, Object> tempMap = (Map<String, Object>) map.get(labelName);
                double difference = Double.parseDouble(String.valueOf(tempMap.getOrDefault("liqDifference", 0D)));
                if (difference < 0) reduceList.add(x);
                else if (difference > 0) increaseList.add(x);
            }
        });
        if (filter < 0) return reduceList;
        else return increaseList;
    }

    @Override
    public Map<String, Object> areaLineChart(AbnormalDynamicVo abnormalDynamicVo) {
        Map<String, Object> resMap = new HashMap<>();
        List<String> ids = getWellIds(abnormalDynamicVo.getOrgId());
        if (ids.isEmpty()) return resMap;
        List<AoidDailyYield> yields = aoidDailyYieldService.getBaseMapper().selectList(new LambdaQueryWrapper<AoidDailyYield>()
                .in(AoidDailyYield::getWellId, ids)
                .ge(AoidDailyYield::getProdDate, DateTimeUtil.formatDateTime(abnormalDynamicVo.getStartDate() + " 00:00:00"))
                .le(AoidDailyYield::getProdDate, DateTimeUtil.formatDateTime(abnormalDynamicVo.getEndDate() + " 00:00:00")));
        Map<LocalDateTime, List<AoidDailyYield>> collect = yields.stream().collect(Collectors.groupingBy(AoidDailyYield::getProdDate));
        for (Map.Entry<LocalDateTime, List<AoidDailyYield>> entry : collect.entrySet()) {
            Map<String, Double> temp = new HashMap<>();
            BigDecimal liqProdDaily = BigDecimal.valueOf(entry.getValue().stream().filter(x -> Objects.nonNull(x.getLiqProdDaily())).mapToDouble(AoidDailyYield::getLiqProdDaily).sum());
            BigDecimal A2LiqProdDaily = BigDecimal.valueOf(entry.getValue().stream().filter(x -> Objects.nonNull(x.getA2LiqProdDaily())).mapToDouble(AoidDailyYield::getA2LiqProdDaily).sum());
            temp.put("liqProdDaily", liqProdDaily.setScale(2, RoundingMode.HALF_UP).doubleValue());
            temp.put("A2LiqProdDaily", A2LiqProdDaily.setScale(2, RoundingMode.HALF_UP).doubleValue());
            resMap.put(DateTimeUtil.formatDateStr(entry.getKey()), temp);
        }
        return resMap.entrySet().stream().sorted(Map.Entry.comparingByKey()).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (k1, k2) -> k1, TreeMap::new));
    }

    @Override
    public Map<String, Object> areaLiqAlarm(String orgId) {
        List<Integer> days = getAllDays(orgId);
        List<AoidLiquidThreshold> list = aoidLiquidThresholdService.getThresholdList(orgId);
        Map<String, Object> resMap = new HashMap<>();
        if (StringUtil.isBlankOrEmpty(orgId)) {
            List<Map<String, String>> allOrg = getDefultAllOrg();
            allOrg.forEach(org -> {
                String id = org.get("orgId");
                for (AoidLiquidThreshold threshold : list) {
                    if (threshold.getAppertain().equals(id)) {
                        resMap.put(id, calLiqAlarm(id, days, threshold.getWorkAreaThreshold()));
                    }
                }
            });
        } else {
            List<Map<String, String>> childrenOrg = getChildrenOrg(orgId);
            for (Map<String, String> org : childrenOrg) {
                for (AoidLiquidThreshold threshold : list) {
                    if (threshold.getAppertain().equals(org.get("parentId"))) {
                        resMap.put(org.get("orgId"), calLiqAlarm(org.get("orgId"), days, threshold.getWorkAreaThreshold()));
                    }
                }
            }
        }
        return resMap;
    }

    private Map<String, Object> calLiqAlarm(String orgId, List<Integer> days, Double threshold) {
        Map<String, Object> temp = new HashMap<>();
        LocalDateTime localDateTime = DateTimeUtil.formatDateTime(LocalDate.now() + " 00:00:00").minusDays(1);
        Double toDayLiq = getLiqByDate(orgId, 0, localDateTime);
        BigDecimal bigDecimal = BigDecimal.valueOf(toDayLiq);
        temp.put("prodDate", DateTimeUtil.formatDateStr(localDateTime));
        temp.put("liq", bigDecimal.setScale(2, RoundingMode.HALF_UP));
        days.forEach(x -> {
            BigDecimal valueOf = BigDecimal.valueOf(toDayLiq - getLiqByDate(orgId, x, localDateTime));
            double scale = valueOf.setScale(2, RoundingMode.HALF_UP).doubleValue();
            if (Math.abs(scale) > toDayLiq * threshold) {
                temp.put(x + "", scale);
            }
        });
        return temp;
    }

    private Double getLiqByDate(String orgId, Integer day, LocalDateTime localDateTime) {
        List<AoidDailyYield> yields = getAoidYields(getWellIds(orgId), localDateTime.minusDays(day), new ArrayList<>());
        return yields.stream().filter(z -> Objects.nonNull(z.getLiqProdDaily())).mapToDouble(AoidDailyYield::getLiqProdDaily).sum();
    }

    private List<Map<String, String>> getChildrenOrg(String orgId) {
        String parAndSonOrg = baseDataClient.getAllOrganizations();
        List<Map<String, String>> list = JSONObject.parseObject(JSONObject.parseObject(parAndSonOrg, Result.class).getData().toString(), List.class);
        return list.stream().filter(x -> x.get("parentId").equals(orgId)).collect(Collectors.toList());
    }

    private List<Map<String, String>> getDefultAllOrg() {
        String parAndSonOrg = baseDataClient.getAllOrganizations();
        List<Map<String, String>> list = JSONObject.parseObject(JSONObject.parseObject(parAndSonOrg, Result.class).getData().toString(), List.class);
        return list.stream().filter(x -> x.get("parentId").equals("0")).collect(Collectors.toList());
    }

    private List<String> getWellIds(String orgId) {
        List<Map<String, String>> wellIds = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getWellListByOrgID(orgId), Result.class).getData().toString(), List.class);
        List<String> ids = new ArrayList<>();
        wellIds.forEach(x -> ids.add(x.get("id")));
        return ids;
    }

    private LambdaQueryWrapper<AoidLiquidAbnormalDynamic> getWrapper(AbnormalDynamicVo abnormalDynamicVo) {
        LambdaQueryWrapper<AoidLiquidAbnormalDynamic> wrapper = new LambdaQueryWrapper<>();
        if (!StringUtil.isBlankOrEmpty(abnormalDynamicVo.getWellId())) {
            wrapper = wrapper.eq(AoidLiquidAbnormalDynamic::getWellId, abnormalDynamicVo.getWellId());
        } else if (Objects.nonNull(abnormalDynamicVo.getWellIds())) {
            wrapper = wrapper.in(AoidLiquidAbnormalDynamic::getWellId, abnormalDynamicVo.getWellIds());
        } else if (!StringUtil.isBlankOrEmpty(abnormalDynamicVo.getOrgName())) {
            wrapper = wrapper.eq(AoidLiquidAbnormalDynamic::getWorkArea, abnormalDynamicVo.getOrgName())
                    .or()
                    .eq(AoidLiquidAbnormalDynamic::getWellArea, abnormalDynamicVo.getOrgName())
                    .or()
                    .eq(AoidLiquidAbnormalDynamic::getStation, abnormalDynamicVo.getOrgName());
        }
        if (Objects.nonNull(abnormalDynamicVo.getStartDate()) && Objects.nonNull(abnormalDynamicVo.getEndDate())) {
            wrapper = wrapper.ge(AoidLiquidAbnormalDynamic::getProdDate, DateTimeUtil.formatDateTime(abnormalDynamicVo.getStartDate() + " 00:00:00"))
                    .le(AoidLiquidAbnormalDynamic::getProdDate, DateTimeUtil.formatDateTime(abnormalDynamicVo.getEndDate() + " 00:00:00"));
        }
        return wrapper.orderByAsc(AoidLiquidAbnormalDynamic::getProdDate);
    }

    private AoidLiquidAbnormalDynamic saveLiquidAbnormal(LocalDateTime date, List<AoidLiquidThreshold> thresholds, List<AoidDailyYield> yields, List<String> list) {
        AoidLiquidAbnormalDynamic abnormalDynamic = new AoidLiquidAbnormalDynamic();
        Map<String, Object> map = new HashMap<>();
        List<AoidDailyYield> toDay = yields.stream().filter(x -> date.equals(x.getProdDate())).collect(Collectors.toList());
        if (toDay.isEmpty()) return abnormalDynamic;
        setBaseData(toDay, abnormalDynamic);
        for (String s : list) {
            int day = Integer.parseInt(s);
            for (AoidLiquidThreshold threshold : thresholds) {
                Map<String, Object> tempMap = new HashMap<>();
                List<AoidDailyYield> temp = yields.stream().filter(x -> abnormalDynamic.getProdDate().minusDays(day).equals(x.getProdDate()) && Objects.nonNull(x.getLiqProdDaily())).collect(Collectors.toList());
                if (temp.isEmpty()) continue;
                if (Objects.isNull(abnormalDynamic.getLiqProdDaily())) continue;
                if (abnormalDynamic.getLiqProdDaily() > threshold.getLiquidVolumeStart() && abnormalDynamic.getLiqProdDaily() < threshold.getLiquidVolumeEnd()) {
                    if (Math.abs(abnormalDynamic.getLiqProdDaily() - temp.get(0).getLiqProdDaily()) > threshold.getThresholdLiq()) {
                        BigDecimal bigDecimal = BigDecimal.valueOf(abnormalDynamic.getLiqProdDaily() - temp.get(0).getLiqProdDaily());
                        tempMap.put("liqDifference", bigDecimal.setScale(2, RoundingMode.HALF_UP));
                        tempMap.put("origin", temp.get(0).getZdjg());
                        map.put(s, tempMap);
                    }
                }
            }
            abnormalDynamic.setLiquidInfo(JSON.toJSONString(map));
        }
        return abnormalDynamic;
    }

    private void setBaseData(List<AoidDailyYield> toDay, AoidLiquidAbnormalDynamic abnormalDynamic) {
        AoidDailyYield yield = toDay.get(0);
        abnormalDynamic.setProdDate(yield.getProdDate())
                .setWellId(yield.getWellId())
                .setLiqProdDaily(yield.getLiqProdDaily())
                .setOilMouth(yield.getYz())
                .setPumpDiameter(yield.getBj())
                .setOilPressure(yield.getYy())
                .setSheathPressure(yield.getTy())
                .setBackPressure(yield.getHy())
                .setOrigin(yield.getZdjg());
    }

    private void setAreaData(List<AoidLiquidAbnormalDynamic> abnormalDynamics) {
        if (abnormalDynamics.isEmpty()) return;
        List<String> ids = new ArrayList<>();
        abnormalDynamics.forEach(x -> {
            ids.add(x.getWellId());
        });
        List<Map<String, Object>> list = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getWellInfoByList(ids), Result.class).getData().toString(), List.class);
        for (Map<String, Object> datum : list) {
            for (AoidLiquidAbnormalDynamic dynamic : abnormalDynamics) {
                if (datum.get("wellId").equals(dynamic.getWellId())) {
                    String[] stationIds = String.valueOf(datum.get("stationId")).split("@");
                    dynamic.setWorkArea(stationIds[0]);
                    dynamic.setWellArea(stationIds[1]);
                    dynamic.setStation(stationIds[2]);
                }
            }
        }
    }

    private List<AoidDailyYield> getAoidYields(List<String> wellIds, LocalDateTime date, List<String> list) {
        boolean present = list.stream().max(Comparator.comparingInt(Integer::parseInt)).isPresent();
        int maxDay = 0;
        if (present) {
            maxDay = Integer.parseInt(list.stream().max(Comparator.comparingInt(Integer::parseInt)).get());
        }
        return aoidDailyYieldService.getBaseMapper().selectList(new LambdaQueryWrapper<AoidDailyYield>()
                .in(AoidDailyYield::getWellId, wellIds)
                .ge(AoidDailyYield::getProdDate, date.minusDays(maxDay))
                .le(AoidDailyYield::getProdDate, date));
    }

    private List<Integer> getAllDays(String orgId) {
        List<AoidLiquidThreshold> thresholdList = aoidLiquidThresholdService.getThresholdList(orgId);
        Set<String> days = new HashSet<>();
        thresholdList.forEach(x -> {
            List<String> list = JSONObject.parseObject(x.getRequirement(), List.class);
            days.addAll(list);
        });
        List<Integer> list = new ArrayList<>();
        days.forEach(x -> list.add(Integer.parseInt(x)));
        return list;
    }
}
