package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidLiquidAbnormal;
import com.gct.aoid.entity.AoidParamSetting;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.mapper.AoidLiquidAbnormalMapper;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidLiquidAbnormalService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2021-12-21
 */
@Service
public class AoidLiquidAbnormalServiceImpl extends ServiceImpl<AoidLiquidAbnormalMapper, AoidLiquidAbnormal> implements AoidLiquidAbnormalService {

    @Autowired
    BaseDataClient baseDataClient;
    @Autowired
    AoidDailyYieldService aoidDailyYieldService;

    @Override
    public Page<AoidLiquidAbnormal> getAoidLiquidAbnormalByPage(JSONObject jsonObject) {
        String time = " 00:00:00";
        JSONArray wellIdSet = jsonObject.getJSONArray("wellIdSet");
        Integer pageSize = jsonObject.getInteger("pageSize");
        Integer pageNum = jsonObject.getInteger("pageNum");
        JSONArray chooseDate = jsonObject.getJSONArray("chooseDate");
        int contrastConditions = jsonObject.getIntValue("contrastConditions");
        QueryWrapper<AoidLiquidAbnormal> wrapper = new QueryWrapper<AoidLiquidAbnormal>();
        if (contrastConditions != 0) {
            if (contrastConditions == 1){
                List<Integer> list = new ArrayList<>();
                list.add(1);
                list.add(4);
                wrapper = wrapper.in("contrast_conditions",list);
            }else {
                wrapper = wrapper.eq("contrast_conditions", contrastConditions);
            }
            wrapper =wrapper.in("well_id", wellIdSet).between("prod_date", DateTimeUtil.formatDateTime(chooseDate.get(0) + time), DateTimeUtil.formatDateTime(chooseDate.get(1) + time).plusDays(1));
        } else {
            wrapper = wrapper.in("well_id", wellIdSet)
                    .between("prod_date", DateTimeUtil.formatDateTime(chooseDate.get(0) + time), DateTimeUtil.formatDateTime(chooseDate.get(1) + time).plusDays(1));
        }
        wrapper = wrapper.ne("fluctuation_amplitude",0)
                         .ne("fluctuation_amplitude::varchar","NaN")
                         .ne("fluctuation_amplitude::varchar","Infinity")
                         .ne("fluctuation_amplitude::varchar","-Infinity'");
        Page<AoidLiquidAbnormal> aoidLiquidAbnormalPage = baseMapper.selectPage(new Page<>(pageNum, pageSize), wrapper);
        for (AoidLiquidAbnormal x : aoidLiquidAbnormalPage.getRecords()) {
            if (Objects.nonNull(x.getFluctuationAmplitude()) &&
                    !Double.isInfinite(x.getFluctuationAmplitude()) && !Double.isNaN(x.getFluctuationAmplitude())) {
                BigDecimal bigDecimal = BigDecimal.valueOf(x.getFluctuationAmplitude()*100);
                x.setFluctuationAmplitude(bigDecimal.setScale(2, RoundingMode.HALF_UP).doubleValue());
            }else x.setFluctuationAmplitude(null);
        }
        return aoidLiquidAbnormalPage;
    }

    @Override
    public boolean addAoidLiquidAbnormalNew(LocalDateTime date, List<AoidParamSetting> paramSettingList) {
        QueryWrapper<AoidDailyYield> wrapper = new QueryWrapper<AoidDailyYield>()
                .ge("prod_date", date.minusDays(30))
                .le("prod_date", date);
        List<AoidDailyYield> aoidDailyYields = aoidDailyYieldService.getBaseMapper().selectList(wrapper);
        Map<String, List<AoidDailyYield>> collect = aoidDailyYields.stream().collect(Collectors.groupingBy(AoidDailyYield::getWellId));
        List<AoidLiquidAbnormal> abnormalList = new ArrayList<>();
        long dailyYieldCount = 0;
        long todayEpochDay = DateTimeUtil.getDateFromDateTime(date).toEpochDay();

        for (Map.Entry<String, List<AoidDailyYield>> singleWellYeidDaily : collect.entrySet()) {
            if (singleWellYeidDaily.getValue() == null || singleWellYeidDaily.getValue().isEmpty()) continue;
            singleWellYeidDaily.getValue().sort((m1, m2) -> m2.getProdDate().compareTo(m1.getProdDate()));
            List<AoidDailyYield> temp = singleWellYeidDaily.getValue();
            AoidLiquidAbnormal abnormal = isToday(todayEpochDay, temp.get(0));
            if (abnormal == null) continue;
            temp.remove(0);
            double avgLiqTemp = 0;//绝对平均
            double avgOilTemp = 0;
            double avgWaterTemp = 0;
            double avgGasTemp = 0;
            Map<String, Double> map = null;
            for (AoidDailyYield item : temp) {
                map = new HashMap<>();
                long days = todayEpochDay - DateTimeUtil.getDateFromDateTime(item.getProdDate()).toEpochDay();
                //计算 平均数据
                if (Objects.isNull(item.getLiqProdDaily())) avgLiqTemp = avgLiqTemp * dailyYieldCount / (dailyYieldCount + 1);
                else avgLiqTemp = avgLiqTemp * dailyYieldCount / (dailyYieldCount + 1) + item.getLiqProdDaily() / (dailyYieldCount + 1);

                if (Objects.isNull(item.getOilProdDaily())) avgOilTemp = avgOilTemp * dailyYieldCount / (dailyYieldCount + 1);
                else avgOilTemp = avgOilTemp * dailyYieldCount / (dailyYieldCount + 1) + item.getOilProdDaily() / (dailyYieldCount + 1);

                if (Objects.isNull(item.getWaterCut()))  avgWaterTemp = avgWaterTemp * dailyYieldCount / (dailyYieldCount + 1);
                else avgWaterTemp = avgWaterTemp * dailyYieldCount / (dailyYieldCount + 1) + item.getWaterCut() / (dailyYieldCount + 1);

                if (Objects.isNull(item.getGasProdDaily())) avgGasTemp = avgGasTemp * dailyYieldCount / (dailyYieldCount + 1);
                else avgGasTemp = avgGasTemp * dailyYieldCount / (dailyYieldCount + 1) + item.getGasProdDaily() / (dailyYieldCount + 1);

                dailyYieldCount++;
                if (days == 1) {
                    abnormal.setYesterdayLiq(item.getLiqProdDaily());
                    abnormal.setYesterdayOil(item.getOilProdDaily());
                    abnormal.setYesterdayWaterCut(item.getWaterCut());
                    abnormal.setYesterdayGas(item.getGasProdDaily());
                }
                if (days <= 11) {
                    abnormal.setTenDayAvgLiq(avgLiqTemp * dailyYieldCount / 10.0);
                    abnormal.setTenDayAvgOil(avgOilTemp * dailyYieldCount / 10.0);
                    abnormal.setTenDayAvgWaterCut(avgWaterTemp * dailyYieldCount / 10.0);
                    abnormal.setTenDayAvgGas(avgGasTemp * dailyYieldCount / 10.0);
                }
                if (days <= 31) {
                    abnormal.setOneMonthAvgLiq(avgLiqTemp * dailyYieldCount / 30.0);//真平均,业务
                    abnormal.setOneMonthAvgOil(avgOilTemp * dailyYieldCount / 10.0);
                    abnormal.setOneMonthAvgWaterCut(avgWaterTemp * dailyYieldCount / 10.0);
                    abnormal.setOneMonthAvgGas(avgGasTemp * dailyYieldCount / 10.0);
                }
                map.put("liq_prod_daily",abnormal.getTodayLiq());
                map.put("water_cut",abnormal.getTodayWaterCut());
                map.put("water_cut_as_yest",abnormal.getYesterdayWaterCut());
                map.put("liq_prod_daily_as_yest",abnormal.getYesterdayLiq());
                map.put("liq_prod_daily_as_month",abnormal.getOneMonthAvgLiq());
                map.put("liq_prod_daily_as_ten",abnormal.getTenDayAvgLiq());
            }
            for (int i = 0; i < paramSettingList.size(); i++) {
                if (StringUtils.isEmpty(map)) break;
                AoidParamSetting aoidParamSetting = paramSettingList.get(i);
                Double aDouble = getDouble(map.get(aoidParamSetting.getSourceCol()));
                Double aDouble1 = getDouble(map.get(aoidParamSetting.getTargetCol()));
                abnormal.setContrastConditions(aoidParamSetting.getId());
                abnormal.setDescription(aoidParamSetting.getReference());
                abnormal.setFluctuationAmplitude((aDouble - aDouble1)/aDouble);
                if (compare(aDouble, aDouble1, aoidParamSetting.getFluctuationAmplitude()) && i > 0) {
                    break;
                }
            }
            abnormalList.add(abnormal);
        }
        getOkAoidLiq(abnormalList);
        if (!abnormalList.isEmpty()) return this.saveOrUpdateBatch(abnormalList,abnormalList.size());
        return false;
    }

    private AoidLiquidAbnormal isToday(long todayEpochDays, AoidDailyYield aoidDailyYield) {
        if (todayEpochDays - aoidDailyYield.getProdDate().toLocalDate().toEpochDay() != 0) return null;

        AoidLiquidAbnormal aoidLiquidAbnormal = new AoidLiquidAbnormal();
        aoidLiquidAbnormal.setWellId(aoidDailyYield.getWellId());
        aoidLiquidAbnormal.setProdDate(aoidDailyYield.getProdDate());
        aoidLiquidAbnormal.setTodayLiq(aoidDailyYield.getLiqProdDaily());
        aoidLiquidAbnormal.setTodayOil(aoidDailyYield.getOilProdDaily());
        aoidLiquidAbnormal.setTodayGas(aoidDailyYield.getGasProdDaily());
        aoidLiquidAbnormal.setTodayWaterCut(aoidDailyYield.getWaterCut());
        aoidLiquidAbnormal.setOilMouth(aoidDailyYield.getYz());
        aoidLiquidAbnormal.setPumpDiameter(aoidDailyYield.getBj());
        aoidLiquidAbnormal.setOilPressure(aoidDailyYield.getYy());
        aoidLiquidAbnormal.setSheathPressure(aoidDailyYield.getTy());
        aoidLiquidAbnormal.setBackPressure(aoidDailyYield.getHy());
        return aoidLiquidAbnormal;
    }

    @Override
    public boolean addAoidLiquidAbnormal(LocalDateTime date, List<AoidParamSetting> paramSettingList) {
        List<Map<String, Object>> liqDailyYestAndTenAndMonth = aoidDailyYieldService.getLiqDailyYestAndTenAndMonth(date);
        if (liqDailyYestAndTenAndMonth.isEmpty()) return false;
        List<AoidLiquidAbnormal> abnormalList = setAoidLiquidAbnormal(liqDailyYestAndTenAndMonth, paramSettingList);
        return this.saveBatch(getOkAoidLiq(abnormalList), abnormalList.size());
    }

    private List<AoidLiquidAbnormal> setAoidLiquidAbnormal(List<Map<String, Object>> liqDailyYestAndTenAndMonth, List<AoidParamSetting> paramSettingList) {
        List<AoidLiquidAbnormal> abnormalList = new ArrayList<>();
        AoidLiquidAbnormal aoidLiquidAbnormal;
        for (Map<String, Object> map : liqDailyYestAndTenAndMonth) {
            {
                aoidLiquidAbnormal = new AoidLiquidAbnormal();
                aoidLiquidAbnormal.setWellId(String.valueOf(map.get("well_id")));
                aoidLiquidAbnormal.setProdDate(DateTimeUtil.formatDateTime(String.valueOf(map.get("prod_date")).substring(0, 19)));
                aoidLiquidAbnormal.setOilMouth(getDouble(map.get("yz")));
                aoidLiquidAbnormal.setPumpDiameter(getDouble(map.get("bj")));
                aoidLiquidAbnormal.setOilPressure(getDouble(map.get("yy")));
                aoidLiquidAbnormal.setSheathPressure(getDouble(map.get("ty")));
                aoidLiquidAbnormal.setBackPressure(getDouble(map.get("hy")));
                aoidLiquidAbnormal.setOneMonthAvgLiq(getDouble(map.get("liq_prod_daily_as_month")));
                aoidLiquidAbnormal.setOneMonthAvgOil(getDouble(map.get("oil_prod_daily_as_month")));
                aoidLiquidAbnormal.setOneMonthAvgWaterCut(getDouble(map.get("water_cut_as_month")));
                aoidLiquidAbnormal.setOneMonthAvgGas(getDouble(map.get("gas_daily_as_month")));
                aoidLiquidAbnormal.setTenDayAvgLiq(getDouble(map.get("liq_prod_daily_as_ten")));
                aoidLiquidAbnormal.setTenDayAvgOil(getDouble(map.get("oil_prod_daily_as_ten")));
                aoidLiquidAbnormal.setTenDayAvgWaterCut(getDouble(map.get("water_cut_as_ten")));
                aoidLiquidAbnormal.setTenDayAvgGas(getDouble(map.get("gas_daily_as_ten")));
                aoidLiquidAbnormal.setYesterdayLiq(getDouble(map.get("liq_prod_daily_as_yest")));
                aoidLiquidAbnormal.setYesterdayOil(getDouble(map.get("oil_prod_daily_as_yest")));
                aoidLiquidAbnormal.setYesterdayWaterCut(getDouble(map.get("water_cut_as_yest")));
                aoidLiquidAbnormal.setYesterdayGas(getDouble(map.get("gas_daily_as_yest")));
                aoidLiquidAbnormal.setTodayLiq(getDouble(map.get("liq_prod_daily")));
                aoidLiquidAbnormal.setTodayOil(getDouble(map.get("oil_prod_daily")));
                aoidLiquidAbnormal.setTodayWaterCut(getDouble(map.get("water_cut")));
                aoidLiquidAbnormal.setTodayGas(getDouble(map.get("gas_prod_daily")));
            }
            for (int i = 0; i < paramSettingList.size(); i++) {
                AoidParamSetting aoidParamSetting = paramSettingList.get(i);
                Double aDouble = getDouble(map.get(aoidParamSetting.getSourceCol()));
                Double aDouble1 = getDouble(map.get(aoidParamSetting.getTargetCol()));
                aoidLiquidAbnormal.setContrastConditions(aoidParamSetting.getId());
                aoidLiquidAbnormal.setDescription(aoidParamSetting.getReference());
                aoidLiquidAbnormal.setFluctuationAmplitude(aDouble - aDouble1);
                if (compare(aDouble, aDouble1, aoidParamSetting.getFluctuationAmplitude()) && i > 0) {
                    break;
                }
            }
            abnormalList.add(aoidLiquidAbnormal);
        }
        return abnormalList;
    }

    private List<AoidLiquidAbnormal> getOkAoidLiq(List<AoidLiquidAbnormal> abnormalList) {
        Set<String> wellId = new HashSet<>();
        abnormalList.forEach(x -> wellId.add(x.getWellId()));
        List<Map<String, Object>> data = (List<Map<String, Object>>) JSONObject.parseObject(baseDataClient.getWellInfoByList(wellId.stream().collect(Collectors.toList())), Result.class).getData();
        for (Map<String, Object> datum : data) {
            for (AoidLiquidAbnormal liquidAbnormal : abnormalList) {
                if (datum.get("wellId").equals(liquidAbnormal.getWellId())) {
                    String[] stationIds = String.valueOf(datum.get("stationId")).split("@");
                    liquidAbnormal.setWorkArea(stationIds[0]);
                    liquidAbnormal.setWellArea(stationIds[1]);
                    liquidAbnormal.setStation(stationIds[2]);
                }
            }
        }
        return abnormalList;
    }

    /**
     * @param source 今天的数据
     * @param target 昨天/十天内/一个月内的数据
     * @param limit  代表数据库中的 波动范围
     */
    private boolean compare(Double source, Double target, Double limit) {
        if (source == null || target == null || limit == null) return false;
        return Math.abs(source - target) > target * limit / 100.0;
    }

    private Double getDouble(Object obj) {
        if (StringUtils.isEmpty(obj)) return 0.0;
        return Double.parseDouble(String.valueOf(obj));
    }
}
