package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.aoid.algorithm.CalDiagramProduction;
import com.gct.aoid.algorithm.CalPumpCard;
import com.gct.aoid.algorithm.CardAreaAlgorithm;
import com.gct.aoid.entity.*;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.AoidSinglewellConfigService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.SGTUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * class name: CalDiagramProductionService
 * @explain 量液标定参数服务类
 * @author 王森玉
 * @version 1.0
 * @since 2022-06-15
 */
@Component
@Slf4j
public class CalibrationParameterService {

    @Autowired
    private BaseDataClient baseDataClient;

    @Autowired
    private AoidDailyYieldService aoidDailyYieldService;

    @Autowired
    private AoidSinglegtYieldService aoidSinglegtYieldService;

    @Autowired
    private CalPumpCard calPumpCard;
    @Autowired
    private CalDiagramProduction calDiagramProduction;

    @Autowired
    private CalDiagramProductionService calDiagramProductionService;

    @Autowired
    private AoidSinglewellConfigService aoidSinglewellConfigService;

    @Autowired
    MutableJobConfigProperties config;

    private final DateTimeFormatter dfValue = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    private final DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");


    /**
     *量液实时运行
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     */
    public Double calYieldEndLiq(String jh,String dateTime) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException, ParseException {

        double rclyxzsum=0.0;
        Double LiqProdDailyEnd = null;

        List<AoidSinglegtYield> resultZdjg = aoidSinglegtYieldService.findByJhbasecs(jh, LocalDateTime.parse(dateTime,df).format(dfValue)+" 08:00:00",
                LocalDateTime.parse(dateTime,df).plusDays(1).format(dfValue)+" 08:00:00");
        if (resultZdjg.size()>0)
        {
            double data[] = new double[resultZdjg.size()];
            List<Double> list = new ArrayList<>();
            for (int i =0; i<resultZdjg.size(); i++)
            {

                AoidMsgValue.BaseParam aoidMsgRealParm = new AoidMsgValue.BaseParam();
                AoidMsgValue.DiagnoseResult diagnoseResultParm = new AoidMsgValue.DiagnoseResult();
                aoidMsgRealParm.setWellId(resultZdjg.get(i).getWellId());
                aoidMsgRealParm.setWellname(resultZdjg.get(i).getWellCommonName());
                aoidMsgRealParm.setProdDate(resultZdjg.get(i).getProdDate());
                aoidMsgRealParm.setS(resultZdjg.get(i).getStroke());
                aoidMsgRealParm.setN(resultZdjg.get(i).getFrequence());
                aoidMsgRealParm.setLMax(resultZdjg.get(i).getSxzh());
                aoidMsgRealParm.setLMin(resultZdjg.get(i).getXxzh());
                aoidMsgRealParm.setSgt(resultZdjg.get(i).getSgt());
                aoidMsgRealParm.setIslegal(resultZdjg.get(i).getSgtIsLegal());
                diagnoseResultParm.setResult2(resultZdjg.get(i).getEczdjg());
                diagnoseResultParm.setResult1(resultZdjg.get(i).getZdjg());
                aoidMsgRealParm.setBj(resultZdjg.get(i).getBj());
                aoidMsgRealParm.setBs(resultZdjg.get(i).getBs());
                aoidMsgRealParm.setHsl(resultZdjg.get(i).getHsl());
                aoidMsgRealParm.setSmd(resultZdjg.get(i).getSmd());
                aoidMsgRealParm.setYmd(resultZdjg.get(i).getYmd());
                aoidMsgRealParm.setQyb(resultZdjg.get(i).getQyb());
                aoidMsgRealParm.setDym(resultZdjg.get(i).getA2Dym());
                AoidMsgValue aoidMsgRealValue  = getAoidMsgReal(aoidMsgRealParm,diagnoseResultParm);
                data[i] += calDiagramProductionService.calEndLiq(aoidMsgRealValue,1.0,0,0, null, 0,
                        0, 0, 0,  0, 0,
                        0, 0, null, null,null,null);




            }
            for (int f=0; f<data.length;f++)
            {
                list.add(data[f]);
                List<Double> filters = new ArrayList<>();
                filters.add(0.0);
                list.removeAll(filters);

            }
            if (list.size()>0)
            {
                for (int p=0;p<list.size();p++)
                {
                    rclyxzsum+=list.get(p);
                }
                LiqProdDailyEnd = rclyxzsum/list.size();
            }
        }


        return LiqProdDailyEnd;

    }



    public AoidMsgValue getAoidMsgReal(AoidMsgValue.BaseParam aoidMsgRealParm, AoidMsgValue.DiagnoseResult diagnoseResultParm)
    {
        AoidMsgValue aoidMsgReal = new AoidMsgValue();
        aoidMsgReal.setBaseParam(aoidMsgRealParm);
        aoidMsgReal.setDiagnoseResult(diagnoseResultParm);
        return aoidMsgReal;
    }


    /**
     *标定参数定时运行
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     */
    public void Calibrationtask() throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException, ParseException {

        double yieldSystemRate =0.0;
        double yieldA2Rate =0.0;
        double diffyieldA2Rate = 0.0;

        String baseParam = baseDataClient.getjhAllConent();
        Result result = JSONObject.parseObject(baseParam, Result.class);
        Object data = result.getData();
        JSONArray jsonArray = JSONArray.parseArray(data.toString());
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jo = jsonArray.getJSONObject(i);
            String wellId = jo.getString("wellId");
            String calibrationList = baseDataClient.selectOneByWellIdList(wellId);
            Result calibrationListresult = JSONObject.parseObject(calibrationList, Result.class);
            Object calibrationObject = calibrationListresult.getData().toString();
            List<Map<String, String>> workAreaDate = config.getCalDailyLiqJob().getParam().getWorkAreaDate();
            String timeValue =null;
            Map<String, String> map = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getOrgInfoByWellID(wellId), Result.class).getData().toString(), Map.class);
            String gfat = map.get("gfat");
            for (Map<String, String> stringStringMap : workAreaDate) {
                if (stringStringMap.get("orgId").equals(gfat)) {
                    timeValue = stringStringMap.getOrDefault("time","08:00:00");
                }
            }
            if (!calibrationObject.equals("[]"))
            {
                String calibrationListjsonValue = StringUtils.strip(calibrationObject.toString(),"[]");
                JSONObject calibrationListjson = JSONObject.parseObject(calibrationListjsonValue);
                if (calibrationListjson.size()>0) {
                    String markedParmExplain = calibrationListjson.getString("markedParmExplain");
                    String markedBy = calibrationListjson.getString("markedBy");
                    JSONObject jsonObject = JSONObject.parseObject(markedParmExplain);
                    String count = jsonObject.getString("count");

                    if (!markedBy.equals("zdcyc"))
                    {
                        if (count.equals("null"))
                        {
                            calibrationprarm(wellId);
                        }
                        else if (Double.parseDouble(count)<=7)
                        {
                            String date =jsonObject.getString("date");
                            String trend = jsonObject.getString("trend");
                            if (Double.parseDouble(count) == 7)
                            {
                                calibrationprarm(wellId);
                            }
                            else {
                                A2Basedata aoidDailyA2One = getA2Basedata(wellId, LocalDateTime.parse(date,df).format(df));
                                A2Basedata aoidDailyA2New = getA2Basedata(wellId, LocalDateTime.now().format(dfValue)+" 00:00:00");
                                Map<String, Object> processparamValue = new HashMap<>();

                                if (aoidDailyA2New.LiqProdDaily!=0 && aoidDailyA2One.LiqProdDaily!=0)
                                {
                                    yieldA2Rate =(aoidDailyA2New.LiqProdDaily-aoidDailyA2One.LiqProdDaily)/aoidDailyA2One.LiqProdDaily;

                                    if (trend.equals("下降"))
                                    {
                                        if (yieldA2Rate<=-0.1){
                                            processparamValue.put("explain", "null");
                                            processparamValue.put("date","null");
                                            processparamValue.put("count", "null");
                                            processparamValue.put("trend", "null");
                                            addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                        }
                                        else
                                        {
                                            processparamValue.put("explain", "需标定");
                                            processparamValue.put("date",LocalDateTime.parse(date,df).format(df));
                                            processparamValue.put("count", Integer.valueOf(count)+1);
                                            processparamValue.put("trend", "下降");
                                            addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                        }
                                    }
                                    else if (trend.equals("上升"))
                                    {
                                        if (yieldA2Rate>=0.1){
                                            processparamValue.put("explain", "null");
                                            processparamValue.put("date","null");
                                            processparamValue.put("count", "null");
                                            processparamValue.put("trend", "null");
                                            addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                        }
                                        else
                                        {
                                            processparamValue.put("explain", "需标定");
                                            processparamValue.put("date",LocalDateTime.parse(date,df).format(df));
                                            processparamValue.put("count", Integer.valueOf(count)+1);
                                            processparamValue.put("trend", "上升");
                                            addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                        }
                                    }
                                    else if (trend.equals("稳定"))
                                    {
                                        if (Math.abs(yieldA2Rate)<=0.1){
                                            processparamValue.put("explain", "null");
                                            processparamValue.put("date","null");
                                            processparamValue.put("count", "null");
                                            processparamValue.put("trend", "null");
                                            addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                        }
                                        else
                                        {
                                            processparamValue.put("explain", "需标定");
                                            processparamValue.put("date",LocalDateTime.parse(date,df).format(df));
                                            processparamValue.put("count", Integer.valueOf(count)+1);
                                            processparamValue.put("trend", "稳定");
                                            addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                        }
                                    }
                                }
                                else {
                                    processparamValue.put("explain", "需标定");
                                    processparamValue.put("date",LocalDateTime.parse(date,df).format(df));
                                    processparamValue.put("count", Integer.valueOf(count)+1);
                                    processparamValue.put("trend", trend);
                                    addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparamValue),"1");
                                }

                            }
                        }
                    }
                }
            }
            else {
                Double aoidDailySystemOne = calYieldEndLiq(wellId, LocalDateTime.now().minusDays(2).format(dfValue)+" 00:00:00");
                Double aoidDailySystemTwo = calYieldEndLiq(wellId, LocalDateTime.now().minusDays(1).format(dfValue)+" 00:00:00");

                A2Basedata aoidDailyA2One = getA2Basedata(wellId, LocalDateTime.now().minusDays(1).format(dfValue)+" 00:00:00");
                A2Basedata aoidDailyA2Two = getA2Basedata(wellId, LocalDateTime.now().format(dfValue)+" 00:00:00");

                if (aoidDailySystemOne!=null && aoidDailySystemTwo!=null)
                {
                    if (aoidDailyA2Two.LiqProdDaily!=0 && aoidDailyA2One.LiqProdDaily!=0)
                    {
                        diffyieldA2Rate = Math.abs(aoidDailyA2Two.LiqProdDaily-aoidDailyA2One.LiqProdDaily);


                        yieldSystemRate =(aoidDailySystemTwo.doubleValue()-aoidDailySystemOne.doubleValue())/aoidDailySystemOne.doubleValue();
                        yieldA2Rate =(aoidDailyA2Two.LiqProdDaily-aoidDailyA2One.LiqProdDaily)/aoidDailyA2One.LiqProdDaily;
                        Map<String, Object> processparam = new HashMap<>();
                        processparam.put("explain", "null");
                        processparam.put("date", "null");
                        processparam.put("count", "null");

                        if (yieldSystemRate<=-0.1 )
                        {
                            if (yieldA2Rate<=-0.1)
                            {
                                addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparam),"1");
                            }

                        }
                        else if (yieldSystemRate>=0.1 )
                        {
                            if (yieldA2Rate>=0.1)
                            {
                                addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparam),"1");
                            }

                        }
                        else if (Math.abs(yieldSystemRate)<=0.1)
                        {
                            if (Math.abs(yieldA2Rate)<=0.1 || diffyieldA2Rate<1)
                            {
                                addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparam),"1");
                            }


                        }
                    }


                }
            }


        }


    }


    public void calibrationprarm(String wellId) throws ParseException, InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        double yieldSystemRate =0.0;
        double yieldA2Rate =0.0;
        double diffyieldA2Rate = 0.0;

        Double aoidDailySystemOne = calYieldEndLiq(wellId, LocalDateTime.now().minusDays(2).format(dfValue)+" 00:00:00");
        Double aoidDailySystemTwo = calYieldEndLiq(wellId, LocalDateTime.now().minusDays(1).format(dfValue)+" 00:00:00");

        A2Basedata aoidDailyA2One = getA2Basedata(wellId, LocalDateTime.now().minusDays(1).format(dfValue)+" 00:00:00");
        A2Basedata aoidDailyA2Two = getA2Basedata(wellId, LocalDateTime.now().format(dfValue)+" 00:00:00");

        if (aoidDailySystemOne!=null && aoidDailySystemTwo!=null)
        {
            if (aoidDailyA2Two.LiqProdDaily!=0 && aoidDailyA2One.LiqProdDaily!=0)
            {
                diffyieldA2Rate = Math.abs(aoidDailyA2Two.LiqProdDaily-aoidDailyA2One.LiqProdDaily);


                yieldSystemRate =(aoidDailySystemTwo.doubleValue()-aoidDailySystemOne.doubleValue())/aoidDailySystemOne.doubleValue();
                yieldA2Rate =(aoidDailyA2Two.LiqProdDaily-aoidDailyA2One.LiqProdDaily)/aoidDailyA2One.LiqProdDaily;
                Map<String, Object> processparam = new HashMap<>();
                processparam.put("explain", "null");
                processparam.put("date", "null");
                processparam.put("count", "null");

                if (yieldSystemRate<=-0.1 )
                {
                    if (yieldA2Rate<=-0.1)
                    {
                        addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparam),"1");
                    }

                }
                else if (yieldSystemRate>=0.1 )
                {
                    if (yieldA2Rate>=0.1)
                    {
                        addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparam),"1");
                    }

                }
                else if (Math.abs(yieldSystemRate)<=0.1)
                {
                    if (Math.abs(yieldA2Rate)<=0.1 || diffyieldA2Rate<1)
                    {
                        addCbPcDevWellAttr(wellId,LocalDateTime.now().format(dfValue)+" 00:00:00",JSONObject.toJSONString(processparam),"1");
                    }


                }
            }


        }
    }








    //获取A2基础数据
    public A2Basedata getA2Basedata(String wellId, String A2ProdDate)throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException
    {

        A2Basedata a2Basedata = new A2Basedata();
        ProDataForCalProduction a2DataVolYesterday;


        String a2dataStrVolYesterdaycalibration = baseDataClient.getWellMeterTeamForCalProduction(wellId, A2ProdDate);
        Result calibrationListresult = JSONObject.parseObject(a2dataStrVolYesterdaycalibration, Result.class);
        Object calibrationObject = calibrationListresult.getData();

        if (calibrationObject!=null)
        {

            JSONObject jo = (JSONObject) JSON.toJSON(calibrationObject);
            double testLiquVolDaily = jo.getDoubleValue("testLiquVolDaily");

            a2Basedata.LiqProdDaily = testLiquVolDaily;
        }
        else {
            String a2dataStrVolYesterday = baseDataClient.getA2VolDataForCalProduction(wellId, A2ProdDate);
            a2DataVolYesterday = (ProDataForCalProduction) JSONObject.parseObject(a2dataStrVolYesterday, Result.class).toFiguredOutObject(ProDataForCalProduction.class);

            if (a2DataVolYesterday!=null)
            {
                if (a2DataVolYesterday.getLiqProdDaily()!=null)
                {
                    a2Basedata.LiqProdDaily = a2DataVolYesterday.getLiqProdDaily();
                }
                else {
                    a2Basedata.LiqProdDaily =0.0;
                }

            }
        }


        return a2Basedata;
    }


    //A2基础数据类
    public class A2Basedata
    {
        private Double LiqProdDaily;
    }



    //添加标定记录
    public  WellAttr addCbPcDevWellAttr(String wellId,String UpdateTime,String MarkedParmExplain,String falg) throws InvocationTargetException, IllegalAccessException, InstantiationException, NoSuchMethodException {
        WellAttr wellAttr = new WellAttr();

        List<Map<String, String>> workAreaDate = config.getCalDailyLiqJob().getParam().getWorkAreaDate();
        String timeValue =null;
        Map<String, String> map = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getOrgInfoByWellID(wellId), Result.class).getData().toString(), Map.class);
        String gfat = map.get("gfat");
        for (Map<String, String> stringStringMap : workAreaDate) {
            if (stringStringMap.get("orgId").equals(gfat)) {
                timeValue = stringStringMap.getOrDefault("time","08:00:00");
            }
        }

        if (falg.equals("1"))
        {
            wellAttr.setWellId(wellId);
            wellAttr.setOilProductsSorting("1");
            if (MarkedParmExplain.contains("标定"))
            {
                wellAttr.setCoefficient(1.0);
            }
            else {
                A2Basedata aoidDailyValue = getA2Basedata(wellId,LocalDateTime.parse(UpdateTime,df).format(df));
                String dateStartCoefficient = LocalDateTime.parse(UpdateTime,df).minusDays(2).format(dfValue)+" "+timeValue;
                String dateEndCoefficient = LocalDateTime.parse(UpdateTime,df).minusDays(1).format(dfValue)+" "+timeValue;
                List<AoidSinglegtYield> resultLiq = aoidSinglegtYieldService.findByJhbasecs(wellId,dateStartCoefficient,dateEndCoefficient);
                if (aoidDailyValue.LiqProdDaily!=0)
                {
                    if (resultLiq.size()>0)
                    {
                        double sumArea =0;
                        for (int y = 0; y < resultLiq.size(); y++) {

                            List<List<Double>> sgtList = new ArrayList<>();
                            if (Objects.nonNull(resultLiq.get(y).getSgt())) {
                                sgtList = SGTUtil.encodeToUseList((String) resultLiq.get(y).getSgt());
                                if (Objects.nonNull(sgtList) && sgtList.size() > 1) {
                                    sgtList.remove(sgtList.size() - 1);
                                    sgtList.add(sgtList.get(0));
                                }
                                Double[][] sgtValue = JSONObject.parseObject(sgtList.toString(), Double[][].class);
                                double areaReal = CardAreaAlgorithm.calArea(sgtValue);
                                if (areaReal>0)
                                {
                                    sumArea += areaReal;
                                }
                            }

                        }
                        double areaAvg = sumArea/resultLiq.size();
                        if (areaAvg<200)
                        {
                            wellAttr.setCoefficient(coefficientValue(aoidDailyValue.LiqProdDaily,resultLiq));
                        }

                    }
                }

            }

            wellAttr.setIsmatching("是");
            wellAttr.setUpdateTime(UpdateTime);
            wellAttr.setMarkedBy("系统自动");
            wellAttr.setMarkedParmExplain(MarkedParmExplain);
        }
        else if (falg.equals("2"))
        {
            wellAttr.setWellId(wellId);
            wellAttr.setOilProductsSorting("1");
            wellAttr.setCoefficient(1.0);
            wellAttr.setIsmatching("是");
            wellAttr.setUpdateTime(UpdateTime);
            wellAttr.setMarkedBy("系统自动");
            wellAttr.setMarkedParmExplain(MarkedParmExplain);
        }
        else if (falg.equals("3"))
        {
            wellAttr.setWellId(wellId);
            wellAttr.setOilProductsSorting("1");

            A2Basedata aoidDailyValue = getA2Basedata(wellId,LocalDateTime.parse(UpdateTime,df).format(df));
            String dateStartCoefficient = LocalDateTime.parse(UpdateTime,df).minusDays(2).format(dfValue)+" "+timeValue;
            String dateEndCoefficient = LocalDateTime.parse(UpdateTime,df).minusDays(1).format(dfValue)+" "+timeValue;
            List<AoidSinglegtYield> resultLiq = aoidSinglegtYieldService.findByJhbasecs(wellId,dateStartCoefficient,dateEndCoefficient);
            if (aoidDailyValue.LiqProdDaily!=0)
            {

                if (resultLiq.size()>0)
                {
                    double sumArea =0;
                    for (int y = 0; y < resultLiq.size(); y++) {

                        List<List<Double>> sgtList = new ArrayList<>();
                        if (Objects.nonNull(resultLiq.get(y).getSgt())) {
                            sgtList = SGTUtil.encodeToUseList((String) resultLiq.get(y).getSgt());
                            if (Objects.nonNull(sgtList) && sgtList.size() > 1) {
                                sgtList.remove(sgtList.size() - 1);
                                sgtList.add(sgtList.get(0));
                            }
                            Double[][] sgtValue = JSONObject.parseObject(sgtList.toString(), Double[][].class);
                            double areaReal = CardAreaAlgorithm.calArea(sgtValue);
                            if (areaReal>0)
                            {
                                sumArea += areaReal;
                            }
                        }

                    }
                    double areaAvg = sumArea/resultLiq.size();

                    if (areaAvg<200)
                    {
                        wellAttr.setCoefficient(coefficientValue(aoidDailyValue.LiqProdDaily,resultLiq));
                    }


                }
            }
            wellAttr.setIsmatching("是");
            wellAttr.setUpdateTime(UpdateTime);
            wellAttr.setMarkedBy("系统自动");
            wellAttr.setMarkedParmExplain(MarkedParmExplain);
        }

        baseDataClient.addCbPcDevWellAttr(wellAttr);
        return wellAttr;
    }





    private static final double expectedCoe = 0.03;

    /**
     * 计算系数
     * @param a2
     * @param result1
     * @return
     */
    public  double coefficientValue(double a2, List<AoidSinglegtYield> result1) {

        double u = 0;
        double d = 10;
        double pre_pzxs = 10;
        double xs =1;
        while (d - u > 1e-6) {
            xs = (u + d) / 2;
            double rclyxzsum = 0.0;
            Double data[] = new Double[result1.size()];
            List<Double> list = new ArrayList<>();
            for (int n = 0; n < result1.size(); n++) {
                data[n] = calWrapper(result1.get(n).getWellId(), result1.get(n).getSgt(),
                        result1.get(n).getStroke(), result1.get(n).getSgtIsLegal(), result1.get(n).getFrequence(),
                        result1.get(n).getEczdjg(), result1.get(n).getZdjg(), result1.get(n).getBj(), result1.get(n).getBs(),
                        result1.get(n).getHsl(), result1.get(n).getSmd(), result1.get(n).getYmd(), result1.get(n).getQyb(),
                        result1.get(n).getA2Dym(), xs);
            }
            list= Arrays.stream(data).filter(x->x>0).collect(Collectors.toList());
            if (list.isEmpty()) break;
            for (int p = 0; p < list.size(); p++) {
                rclyxzsum += list.get(p);
            }

            double production = rclyxzsum / list.size();

            if ((Math.abs(xs) < 0.1 || Math.abs(xs) > 10 || Math.abs(pre_pzxs - xs) <= expectedCoe) &&Math.abs(production - a2)<a2*expectedCoe) {
                break;
            }
            if (a2 - production > 0 ) {
                u = xs;
            } else if (production - a2 > 0) {
                d = xs;
            }
            pre_pzxs = xs;
        }
        return xs;


    }


    /**
     * 量液入口
     * @param wellid
     * @param sgt
     * @param stroke
     * @param sgtIsLegal
     * @param frequence
     * @param eczdjg
     * @param zdjg
     * @param bj
     * @param bs
     * @param hsl
     * @param smd
     * @param ymd
     * @param qyb
     * @param dym
     * @param pzxs
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws NoSuchMethodException
     */

    public double calWrapper(String wellid,String sgt, double stroke, int sgtIsLegal,double frequence, String eczdjg,String zdjg,double
            bj,double bs,double hsl,double smd,double ymd,double qyb,double dym,double pzxs)   {
        double rcyl1 = 0;
        rcyl1 = cal(sgt,stroke,sgtIsLegal,frequence,eczdjg,zdjg,bj,bs,hsl,smd,ymd,qyb,dym,pzxs);
        return   rcyl1;

    }

    /**
     * 稀油量液
     * @param sgt1
     * @param stroke
     * @param sgtIsLegal
     * @param frequence
     * @param eczdjg
     * @param zdjg
     * @param bj
     * @param bs
     * @param hsl
     * @param smd
     * @param ymd
     * @param qyb
     * @param dym
     * @param pzxs
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    public double cal(String sgt1, double stroke, int sgtIsLegal,double frequence, String eczdjg,String zdjg,double
            bj,double bs,double hsl,double smd,double ymd,double qyb,double dym,double pzxs) {


        double rcyl1 =0;
        AoidSinglewellConfig effectStrockModel = aoidSinglewellConfigService.getConfigOneObjByID(1);
        Double[][] bgt = null;
        Double[][] sgt = sgt1 == null ? null : SGTUtil.encodeToDoubleArray(sgt1);

        if(!"-1".equals(zdjg))
        {
            bgt = calPumpCard.cal_bgt(sgt, stroke, sgtIsLegal, 1.0, eczdjg);

            if (bgt!=null)
            {
                CalDiagramProduction.CalProductionResult production = new CalDiagramProduction.CalProductionResult();
                production = calDiagramProduction.calProduction(
                        sgt, bgt, stroke, frequence, eczdjg, bj, bs, hsl, smd, ymd, qyb, dym, pzxs, effectStrockModel
                );
                rcyl1= production.rcyl1;
            }
        }
        return  rcyl1;
        //endregion
    }

}
