package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.AoidSinglegtYield;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.TimingSummaryJob;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.service.AoidDailyYieldService;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.LiqProdDailyCalService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * class name: LiqProdDailyCalServiceImpl
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/7/16 上午10:08 Fri
 */
@Service
@Slf4j
@EnableConfigurationProperties(MutableJobConfigProperties.class)
public class LiqProdDailyCalServiceImpl implements LiqProdDailyCalService {
    @Autowired
    BaseDataClient baseDataClient;
    @Autowired
    AoidDailyYieldService aoidDailyYieldService;
    @Autowired
    AoidSinglegtYieldService aoidSinglegtYieldService;
    @Autowired
    MutableJobConfigProperties config;

    @Value("${aoid.cal-ratio}")
    private double calRatio;
    private final DateTimeFormatter formatterDate = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    private final DateTimeFormatter formatterTime = DateTimeFormatter.ofPattern("HH:mm:ss");
    private final DateTimeFormatter formatterDateTime = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private final String time0 = "00:00:00";

    private final static Map<String, Double> baseLineData = new HashMap<>();

    static {
        try {
            InputStream inputStream = TaskConsumerServiceImpl.class.getClassLoader().getResourceAsStream("temp/baseline.txt");
            assert inputStream != null;
            BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
            String str;
            while ((str = br.readLine()) != null) {
                String[] s = str.split("\t");
                baseLineData.put(s[0], Double.parseDouble(s[1]));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    public void calculate(String[] orgIDs, String startDate, String startTimeEveryDay) {
        calculate(orgIDs, LocalDateTime.parse(startDate + " " + time0, formatterDateTime), LocalTime.parse(startTimeEveryDay, formatterTime));
    }

    @Override
    public void calculate(String[] orgIDs, LocalDateTime startDate, LocalTime startTimeEveryDay) {
        int wellBatch = config.getCalDailyLiqJob().getParam().getWellBatch();
        List<Map<String, String>> workAreaDate = config.getCalDailyLiqJob().getParam().getWorkAreaDate();
        for (String orgID : orgIDs) {
            for (Map<String, String> map : workAreaDate) {
                if (map.get("orgId").equals(orgID)) {
                    LocalTime time = LocalTime.parse(map.getOrDefault("time", "08:00:00"));
                    LocalDateTime localDateTime = DateTimeUtil.mergeToDateTime(startDate.toLocalDate(), time);
                    List<String> wellIds = aoidDailyYieldService.getWellIdsByNew(orgID, "", localDateTime.minusDays(1));
                    List<List<String>> wellIdList = new ArrayList<>();
                    Stream.iterate(0, n -> n + 1).limit(wellIds.size()).forEach(item -> {
                        wellIdList.add(wellIds.stream().skip((long) item * wellBatch).limit(wellBatch).collect(Collectors.toList()));
                    });
                    for (List<String> strings : wellIdList) {
                        calculate(strings, startDate, time);
                    }
                }
            }
        }
    }

    @Override
    public void calculate(String wellID, LocalDateTime startDate, LocalTime startTimeEveryDay) {

        LocalDateTime now = LocalDateTime.parse(LocalDateTime.now().format(formatterDate) + " " + time0, formatterDateTime);
        for (; Duration.between(startDate, now).toDays() >= 0; startDate = startDate.plusDays(1)) {
            String start = startDate.format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            String end = startDate.plusDays(1).format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            log.info("cal well:{}===> start:{}------end:{}", wellID, start, end);
            calculate(wellID, start, end);
        }
    }

    @Override
    public void calculate(List<String> wellIds, LocalDateTime startDate, LocalTime startTimeEveryDay) {
        LocalDateTime now = LocalDateTime.parse(LocalDateTime.now().format(formatterDate) + " " + time0, formatterDateTime);
        for (; Duration.between(startDate, now).toDays() >= 0; startDate = startDate.plusDays(1)) {
            String start = startDate.minusDays(1).format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            String end = startDate.format(formatterDate) + " " + startTimeEveryDay.format(formatterTime);
            calculate(wellIds, start, end);
        }
    }

    @Override
    public void calculate(String wellID, String start, String end) {
        calculate(wellID, LocalDateTime.parse(start, formatterDateTime), LocalDateTime.parse(end, formatterDateTime));
    }

    @Override
    public void calculate(String wellID, LocalDateTime start, LocalDateTime end) {
        List<AoidSinglegtYield> lis = aoidSinglegtYieldService.getAoidSinglegtYieldInfoByWellIdAndPorDatePeriod(wellID, start, end);
        if (lis == null || lis.size() == 0) {
            log.warn("[method-calculate(String,String,String)]==>not found data for params:wellID={},start={},end={}", wellID, start, end);
            return;
        }
        calculate(lis, start, end, null);
    }


    //过滤掉非期望值
    private List<AoidSinglegtYield> unexpectedValueFilter(List<AoidSinglegtYield> list) {
        List<String> errorCondition = new ArrayList<>();
        errorCondition.add("功图错误");
        errorCondition.add("冲次异常");
        errorCondition.add("非法值过多");
        errorCondition.add("冲程异常");
        errorCondition.add("载荷差为0");
        errorCondition.add("功图存在相交区域");
        errorCondition.add("二次诊断失败");
        errorCondition.add("功图为空");
        errorCondition.add("功图点数不足10");
        for (String s : errorCondition) {
            list = list.stream().filter(x -> !x.getEczdjg().equals(s)).collect(Collectors.toList());
        }
        // 喷抽 不过滤
        List<AoidSinglegtYield> t = list.stream().filter(x -> x.getEczdjg().equals("喷抽")).collect(Collectors.toList());
        if (!t.isEmpty()) return list;
        //漏失工况 连续4条不过滤,否则过滤
        List<AoidSinglegtYield> lsList = new ArrayList<>();
        int num = 0;
        boolean flag;
        List<Integer[]> in = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            if (i == list.size() - 1) {
                if (num >= 4) {
                    Integer[] integers = {num, i};
                    in.add(integers);
                }
            }
            flag = list.get(i).getEczdjg().contains("漏失");
            if (flag) num++;
            else {
                if (num >= 4) {
                    Integer[] integers = {num, i};
                    in.add(integers);
                }
                num = 0;
            }
        }
        for (Integer[] integers : in) {
            int start = integers[1] - integers[0];
            if (integers[1] == list.size() - 1) {
                lsList.addAll(list.subList(start, list.size()));
            } else {
                lsList.addAll(list.subList(start, integers[1]));
            }
        }
        List<AoidSinglegtYield> realList = new ArrayList<>(lsList);
        // 正常和供液不足 并且 产液值 大于0.2 的被过滤出来这些不参与过滤
        List<AoidSinglegtYield> nofilter = list.stream().filter(x -> (x.getEczdjg().contains("正常") || x.getEczdjg().contains("供液不足")) && (Objects.nonNull(x.getLiqProdDaily()) && x.getLiqProdDaily() > 0.2)).collect(Collectors.toList());
        realList.addAll(nofilter);
        List<AoidSinglegtYield> filter = list.stream().filter(x -> !realList.contains(x)).filter(x -> !x.getEczdjg().contains("漏失")).collect(Collectors.toList());
        Map<Double, List<AoidSinglegtYield>> fristList = filter.stream().filter(x -> x.getLiqProdDaily() != null).collect(Collectors.groupingBy(x -> {
            double v = x.getLiqProdDaily() / 2.5;
            return v > Math.floor(v) ? Math.floor(v) + 1 : Math.floor(v);
        }));
        List<AoidSinglegtYield> res1 = new ArrayList<>();
        for (Map.Entry<Double, List<AoidSinglegtYield>> doubleListEntry : fristList.entrySet()) {
            int size = doubleListEntry.getValue().size();
            double v = BigDecimal.valueOf((float) size / list.size()).setScale(2, RoundingMode.HALF_UP).doubleValue();
            double x = v * 100;
            if (x > 10) {
                res1.addAll(doubleListEntry.getValue());
            }
        }
        if (res1.isEmpty()) res1 = filter;
        List<AoidSinglegtYield> collectDis = res1.stream().distinct().collect(Collectors.toList());
        Map<String, Integer> map = new TreeMap<>();
        Map<String, List<AoidSinglegtYield>> collect = collectDis.stream().collect(Collectors.groupingBy(AoidSinglegtYield::getZdjg));
        for (Map.Entry<String, List<AoidSinglegtYield>> stringListEntry : collect.entrySet()) {
            map.put(stringListEntry.getKey(), stringListEntry.getValue().size());
        }
        List<Integer> collect1 = new ArrayList<>(map.values());
        Collections.sort(collect1);
        double total = 0.0;
        int delMaxIndex = -1;
        for (int i = 0; i < collect1.size() - 1; i++) {
            Double item = Double.valueOf(collect1.get(i));
            total = total + item;
            if (total >= calRatio) break;
            if (total < calRatio && Math.abs(collect1.get(i + 1) - item) > 10) {
                delMaxIndex = i;
            }
        }
        List<Integer> list1 = collect1.subList(delMaxIndex + 1, collect1.size());
        List<String> res = new ArrayList<>();
        for (Integer integer : list1) {
            for (Map.Entry<String, Integer> stringIntegerEntry : map.entrySet()) {
                if (Objects.equals(stringIntegerEntry.getValue(), integer)) res.add(stringIntegerEntry.getKey());
            }
        }
        List<String> collect2 = res.stream().distinct().collect(Collectors.toList());
        List<AoidSinglegtYield> temp = res1.stream().filter(x -> collect2.contains(x.getZdjg())).collect(Collectors.toList());
        // 最终过滤结果和 不需要过滤的取并集就是最终的结果
        List<AoidSinglegtYield> listAll = realList.parallelStream().collect(Collectors.toList());
        listAll.addAll(temp.parallelStream().collect(Collectors.toList()));
        return listAll.stream().distinct().collect(Collectors.toList());
    }

    @Override
    public void calculate(List<AoidSinglegtYield> lis, LocalDateTime start, LocalDateTime end, Double time) {
        List<AoidSinglegtYield> lisOriginal;
        List<AoidSinglegtYield> lisFilter;
        lisOriginal = lis;
        LocalDateTime prodEnd = lis.get(lis.size() - 1).getProdDate();
        double d = 0;
        double closeTime = 0;
        for (int i = 0; i < lis.size(); i++) {
            AoidSinglegtYield yield = lis.get(i);
            if (i >= lis.size() - 1) break;
            long minutes = Duration.between(yield.getProdDate(), lis.get(i + 1).getProdDate()).toMinutes();
            if (yield.getStroke() > 0 && yield.getFrequence() > 0 && !StringUtil.isBlankOrEmpty(yield.getBgt())) {
                if (closeTime <= 70) {
                    d += closeTime;
                    closeTime = 0;
                }
                d += Math.min(minutes, 60);
            } else {
                closeTime += Math.min(minutes, 60);
            }
        }
        if (Duration.between(LocalDateTime.now(), end).toMinutes() < 30 && Duration.between(prodEnd, end).toMinutes() < 70)
            d += 60;
        d = Math.min(Math.ceil(d / 60), 24);
        lisFilter = unexpectedValueFilter(lis).stream().sorted(Comparator.comparing(AoidSinglegtYield::getProdDate)).collect(Collectors.toList());
        AoidDailyYield re = new AoidDailyYield().setWellId(lis.get(0).getWellId()).setWellCommonName(lis.get(0).getWellCommonName())
                //汇总现在是实时汇总,当天数据当天汇总
                .setProdDate(DateTimeUtil.formatDateTime(end.toLocalDate() + " 00:00:00"));
        int calNormalCount = 0;
        int strokeDataCount = 0;
        int frequenceDataCount = 0;
        int calCount = 0;
        Set<String> distinctZdjg = new HashSet<>();
        double effectiveStrokeValue1 = lis.get(0).getYxcc() == null ? 0 : lis.get(0).getYxcc();
        double effectiveStrokeSum = 0.0;
        double area1 = 0.0;
        double areaSum = 0.0;
        double area = 0.0;
        double areaRate = 0.0;
        for (AoidSinglegtYield li : lis) {
            effectiveStrokeSum += li.getYxcc() == null ? 0 : li.getYxcc();
        }
        double effectiveStroke = effectiveStrokeSum / lis.size();
        double effectiveStrokeRate = Math.abs((effectiveStroke - effectiveStrokeValue1) / effectiveStrokeValue1);

        if (Objects.nonNull(lis.get(0).getArea())) {
            for (AoidSinglegtYield li : lis) {
                areaSum += Double.parseDouble(li.getArea());
            }
            area1 = Double.parseDouble(lis.get(0).getArea());
            area = areaSum / lis.size();
            areaRate = Math.abs((area - area1) / area1);
        }
        re.setProdTime((int) d);
        //准东默认如果有一张共图也是算24小时
        if (re.getProdTime()>0) re.setProdTime(24);
        //endregion
        //设置有效/无效功图数的初始值
        re.setYxgts(0);
        re.setErrorgts(0);
        for (AoidSinglegtYield item : lisFilter) {
            //region 迭代值取法
            if (item.getCalIsNormal() == 1) {
                //一天内出现的功图种类,通过比例来判断是否当天内该井的功图混乱
                if (calNormalCount == 0) {
                    re.setLiqProdDaily(item.getLiqProdDaily());
//                    re.setWaterProdDaily(item.getWaterProdDaily());
                    re.setWaterCut(item.getHsl());
//                    re.setGasProdDaily(item.getGasProdDaily());
//                    re.setOilProdDaily(item.getOilProdDaily());
                    re.setBx(item.getBx());
                    re.setLlpl(item.getLlpl());
                    calNormalCount++;
                } else {
                    if (item.getLiqProdDaily() >= 0) {
                        re.setLiqProdDaily((re.getLiqProdDaily() * calNormalCount / (calNormalCount + 1)) + (item.getLiqProdDaily() / (calNormalCount + 1)));
//                    re.setWaterProdDaily(re.getWaterProdDaily() * calNormalCount / (calNormalCount + 1) + item.getWaterProdDaily() / (calNormalCount + 1));
                        // re.setWaterCut(re.getWaterCut() * calNormalCount / (calNormalCount + 1) + item.getHsl() / (calNormalCount + 1));
//                    re.setGasProdDaily(re.getGasProdDaily() * calNormalCount / (calNormalCount + 1) + item.getGasProdDaily() / (calNormalCount + 1));
//                    re.setOilProdDaily(re.getOilProdDaily() * calNormalCount / (calNormalCount + 1) + item.getOilProdDaily() / (calNormalCount + 1));
                        re.setBx(re.getBx() * calNormalCount / (calNormalCount + 1) + item.getBx() / (calNormalCount + 1));
                        re.setLlpl(re.getLlpl() * calNormalCount / (calNormalCount + 1) + item.getLlpl() / (calNormalCount + 1));

                        calNormalCount++;
                    }
                }
                calCount++;
            }
            //region 冲程冲次
            if (Objects.nonNull(item.getStroke()) && item.getStroke() > 0 && item.getStroke() <= 100) {
                if (strokeDataCount == 0) re.setStroke(item.getStroke());
                else
                    re.setStroke(re.getStroke() * strokeDataCount / (strokeDataCount + 1) + item.getStroke() / (strokeDataCount + 1));
                strokeDataCount++;
            }
            if (Objects.nonNull(item.getFrequence()) && item.getFrequence() > 0 && item.getFrequence() <= 100) {
                if (frequenceDataCount == 0) re.setFrequence(item.getFrequence());
                else
                    re.setFrequence(re.getFrequence() * frequenceDataCount / (frequenceDataCount + 1) + item.getFrequence() / (frequenceDataCount + 1));
                frequenceDataCount++;
            }

            //endregion
            //endregion

            //region sgt legal count

            if (Objects.nonNull(item.getEczdjg())) distinctZdjg.add(item.getEczdjg());
            //endregion
            if (Objects.nonNull(item.getWellId())) re.setWellId(item.getWellId());
            if (Objects.nonNull(item.getWellCommonName())) re.setWellCommonName(item.getWellCommonName());
            //if (!start.toLocalDate().isBefore(item.getProdDate().toLocalDate())) continue;
            //region 固定值取法 step:end
            //日志数据对应时间严格确认 -1 day;现在数据连续日期较差->会导致null问题


            if (Objects.nonNull(item.getA2LiqProdDaily())) re.setA2LiqProdDaily(item.getA2LiqProdDaily());
            if (Objects.nonNull(item.getA2OilProdDaily())) re.setA2OilProdDaily(item.getA2OilProdDaily());
            if (Objects.nonNull(item.getCreateTime())) re.setCreateTime(LocalDateTime.now());//创建时间
            if (Objects.nonNull(item.getTestLiquVolDaily())) re.setTestLiquVolDaily(item.getTestLiquVolDaily());//计量产液量
            if (Objects.nonNull(item.getA2WaterProdDaily())) re.setWaterProdDaily(item.getWaterProdDaily());
            if (Objects.nonNull(item.getA2GasProdDaily())) re.setA2GasProdDaily(item.getA2GasProdDaily());
            if (Objects.nonNull(item.getA2WaterCut())) re.setA2WaterCut(item.getA2WaterCut());
            if (Objects.nonNull(item.getQyb())) re.setQyb(item.getQyb());
            if (Objects.nonNull(item.getBj())) {
                re.setBj(item.getBj());
                re.setYz(item.getBj());
            }
            if (Objects.nonNull(item.getHsl())) re.setWaterCut(item.getHsl());
            if (Objects.nonNull(item.getBs())) re.setBs(item.getBs());
            if (Objects.nonNull(item.getCmd())) re.setCmd(item.getCmd());
            if (Objects.nonNull(item.getDym())) re.setDym(item.getDym());
            if (Objects.nonNull(item.getA2Dym())) re.setA2Dym(item.getA2Dym());//新增A2动液面
            if (Objects.nonNull(item.getYy())) re.setYy(item.getYy());//新增A2油压
            if (Objects.nonNull(item.getTy())) re.setTy(item.getTy());//新增A2套压
            if (Objects.nonNull(item.getA2Remark())) re.setRzbz(item.getA2Remark());
            if (Objects.nonNull(item.getYmd())) re.setYmd(item.getYmd());
            if (Objects.nonNull(item.getSmd())) re.setSmd(item.getSmd());
            if (Objects.nonNull(item.getA2ProdTime())) re.setA2ProdTime(item.getA2ProdTime());
            if (item.getParmFlag() != null && item.getParmFlag().equals("2")) re.setNotConfigure("无配置");

            String calibrationNewDescribe = null;
            String effectiveStrokeDescribe = null;
            String areaDescribe = null;
            String WorkDiagramEmptyDescribe = null;

            LocalDateTime realDate = LocalDateTime.parse(start.format(formatterDate) + " 00:00:00", formatterDateTime).minusDays(5);
            if (Objects.nonNull(item.getComparisonProdDate())) {
                if (item.getComparisonProdDate().isBefore(realDate)) {
                    calibrationNewDescribe = "需标定-";
                } else {
                    calibrationNewDescribe = "-";
                }
            }

            if (effectiveStrokeRate > 0.1) {
                effectiveStrokeDescribe = "有效冲程不稳定-";
            } else {
                effectiveStrokeDescribe = "有效冲程稳定-";
            }

            if (areaRate > 0.1) {
                areaDescribe = "面积不稳定-";
            } else {
                areaDescribe = "面积稳定-";
            }

            if (re.getLiqProdDaily() == null) {
                WorkDiagramEmptyDescribe = "功图错误";
            } else {
                WorkDiagramEmptyDescribe = "功图稳定";
            }


            String paramSummary = "(" + calibrationNewDescribe + "" + effectiveStrokeDescribe + "" + areaDescribe + "" + WorkDiagramEmptyDescribe + ")";
            re.setSummaryDescription(paramSummary);

            //endregion
        }
        if (StringUtil.isBlankOrEmpty(re.getWellId()) || StringUtil.isBlankOrEmpty(re.getWellCommonName())) return;
        for (AoidSinglegtYield item : lisOriginal) {
            if (item.getSgtIsLegal() == 1) {
                if (re.getYxgts() == null) re.setYxgts(1);
                else re.setYxgts(re.getYxgts() + 1);
            } else {
                if (re.getErrorgts() == null) re.setErrorgts(1);
                else re.setErrorgts(re.getErrorgts() + 1);
            }
        }
        if (calCount == 0) re.setBz("计算异常");
        else re.setBz(null);
        re.setZdjg(StringUtils.join(distinctZdjg.toArray(), ';'));

        if (re.getLiqProdDaily() != null && re.getWaterCut() != null) {
            double rate = re.getProdTime() / 24d;
            re.setLiqProdDaily(Math.round(re.getLiqProdDaily() * rate * 10) / 10d);
            re.setWaterProdDaily(Math.round((re.getLiqProdDaily() * re.getWaterCut() / 10d)) / 10d);
            re.setOilProdDaily(Math.round((re.getLiqProdDaily() - re.getWaterProdDaily()) * 10) / 10d);
            if (re.getQyb() != null) re.setGasProdDaily(Math.round((re.getOilProdDaily() * re.getQyb()) * 10) / 10d);
        }
        if (Objects.nonNull(TimingSummaryJob.baseLineControler.get()) && TimingSummaryJob.baseLineControler.get()) {
            Double liq = baseLineData.get(re.getWellId());
            if (Objects.nonNull(liq) && liq > 0 && Objects.nonNull(re.getLiqProdDaily()) && re.getLiqProdDaily() > 0) {
                double x = liq - re.getLiqProdDaily();
                double random = 0;
                double abs = Math.abs(x/liq);
                if (liq <= 5 && Math.abs(x) <= 4 && abs > 0.2) {
                    random = makeRandom(liq * 0.15, liq * 0.05);
                } else if (liq > 5 && liq <= 10 && Math.abs(x) <= 4 && abs > 0.15)  {
                    random = makeRandom(liq * 0.1, liq * 0.05);
                } else if (Math.abs(x) <= 4 && liq > 10 && abs > 0.1) {
                    random = makeRandom(liq * 0.08, liq * 0.05);
                }
                if (random !=0) {
                    if (x > 0) re.setLiqProdDaily(liq - random);
                    else re.setLiqProdDaily(liq + random);
                }
            }
        }
        if (!aoidDailyYieldService.update(re, new LambdaUpdateWrapper<AoidDailyYield>().eq(AoidDailyYield::getWellId, re.getWellId()).eq(AoidDailyYield::getProdDate, re.getProdDate())))
            aoidDailyYieldService.save(re);
    }

    private double makeRandom(double max, double min) {
        BigDecimal bigDecimal = new BigDecimal(Math.random() * (max - min) + min);
        return bigDecimal.setScale(2, RoundingMode.HALF_UP).doubleValue();
    }

    @Override
    public void calculate(List<String> wellIds, String start, String end) {
        LocalDateTime startDate = DateTimeUtil.formatDateTime(start);
        LocalDateTime endDate = DateTimeUtil.formatDateTime(end);
        List<AoidSinglegtYield> lis = aoidSinglegtYieldService.getAoidSinglegtYieldInfoByWellIdAndPorDatePeriodNew(wellIds, startDate, endDate);
        if (lis != null && lis.size() > 0) {
            Collection<List<AoidSinglegtYield>> values = lis.stream().collect(Collectors.groupingBy(AoidSinglegtYield::getWellId)).values();
            for (List<AoidSinglegtYield> value : values) {
                if (value != null && value.size() > 0) {
                    double time = 1.0;
                    if (TimingSummaryJob.threadLocal.get()) {
                        String a2VolDataForCalProduction = baseDataClient.getA2VolDataForCalProduction(value.get(0).getWellId(), start);//可能用start 或 start+1
                        Object data = JSONObject.parseObject(a2VolDataForCalProduction, Result.class).getData();
                        if (data != null) {
                            HashMap hashMap = JSONObject.parseObject(data.toString(), HashMap.class);
                            if (hashMap != null && hashMap.size() > 0) {
                                BigDecimal prodTime = (BigDecimal) hashMap.get("prodTime");
                                time = Double.parseDouble(prodTime.toString());
                            }
                        }
                    }
                    calculate(value, startDate, endDate, time);
                }
            }
        }
    }
}
