package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.aoid.entity.AoidDailyYield;
import com.gct.aoid.entity.YieldErrorAnalysis;
import com.gct.aoid.mapper.YieldErrorAnalysisMapper;
import com.gct.aoid.service.YieldErrorAnalysisService;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jyx
 * @since 2021-01-28
 */
@Service
@Slf4j
public class YieldErrorAnalysisServiceImpl extends ServiceImpl<YieldErrorAnalysisMapper, YieldErrorAnalysis> implements YieldErrorAnalysisService {

    /**
     * 添加误差率数据
     * @param yieldErrorAnalysis
     * @return
     */
    @Override
    public Result addYieldErrorAnalysis(YieldErrorAnalysis yieldErrorAnalysis) {

        int insert = baseMapper.insert(yieldErrorAnalysis);
        return insert==1?Result.create(CodeMsgEnum.ADD_SUCCESS):Result.create(CodeMsgEnum.ADD_FAIL);
    }

    @Override
    public IPage<YieldErrorAnalysis> findWellWc(Integer pageNum, Integer pageSize, String zyq, String wcStart,String wcEnd, String cylStart, String cylEnd,String jh,String  dateStart,String dateEnd) {


        Page<AoidDailyYield> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        Double wcStartFormat = new Double(wcStart);
        Double wcEndFormat = new Double(wcEnd);
        Double cylStartFormat = new Double(cylStart);
        Double cylEndFormat = new Double(cylEnd);


        IPage<YieldErrorAnalysis> page = null;
        if (jh == null)
        {
            page=  baseMapper.findWellWc(paramPage, zyq, wcStartFormat, wcEndFormat, cylStartFormat,cylEndFormat,dateStart,dateEnd);
        }
        else {
            page=  baseMapper.findWellWcJhvalue(paramPage, wcStartFormat, wcEndFormat, cylStartFormat,cylEndFormat,jh,dateStart,dateEnd);
        }

        for (YieldErrorAnalysis record : page.getRecords()) {

            record.setTodayYieldError(100*record.getTodayYieldError());

        }

        return page;
    }


    @Override
    public JSONObject exportExcelYieldError(String zyq, String wcStart,String wcEnd, String cylStart, String cylEnd,String jh,String  dateStart,String dateEnd){
        QueryWrapper<YieldErrorAnalysis> wrapper = new QueryWrapper<YieldErrorAnalysis>();
        wrapper= wrapper.ge("prod_date",DateTimeUtil.formatDateTime(dateStart+" 00:00:00"))
                .le("prod_date",DateTimeUtil.formatDateTime(dateEnd+" 00:00:00"))
                .ge("today_yield_error",Double.parseDouble(wcStart))
                .le("today_yield_error",Double.parseDouble(wcEnd))
                .ge("today_a2_liq_prod_daily",Double.parseDouble(cylStart))
                .le("today_a2_liq_prod_daily",Double.parseDouble(cylEnd))
                .orderByDesc("today_yield_error");
        if (jh!=null)
        {
            wrapper.eq("well_id",jh);
        }
        else {
            wrapper.eq("zyq",zyq);
        }
        List<YieldErrorAnalysis> aoidDailyYields = baseMapper.selectList(wrapper);
        JSONObject jo = new JSONObject();
        jo.put("data", dataProcessing(aoidDailyYields));
        return jo;
    }

    private JSONArray dataProcessing(List<YieldErrorAnalysis> yieldErrorAnalysisList ){
        JSONArray data = new JSONArray();
        List<YieldErrorAnalysis> collect = yieldErrorAnalysisList.stream().sorted(Comparator.comparing(YieldErrorAnalysis::getProdDate)).collect(toList());
        collect.forEach(x -> {
            JSONObject rowObj = (JSONObject) JSONObject.toJSON(x);
            if (rowObj.get("prodDate")!=null){
                rowObj.put("prodDate",rowObj.get("prodDate").toString().substring(0,10));
            }
            if (rowObj.get("calibrationNew")!=null){
                rowObj.put("calibrationNew",rowObj.get("calibrationNew").toString().substring(0,10));
            }
            data.add(rowObj);
        });
        return data;
    }




}
