package com.gct.aoid.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.gct.aoid.algorithm.CalDiagramProduction;
import com.gct.aoid.algorithm.CalPumpCard;
import com.gct.aoid.algorithm.CardAreaAlgorithm;
import com.gct.aoid.entity.AoidGkModel;
import com.gct.aoid.entity.AoidSinglegtYield;
import com.gct.aoid.entity.AoidSinglewellConfig;
import com.gct.aoid.entity.MQ.*;
import com.gct.aoid.entity.ProDataForCalProduction;
import com.gct.aoid.feignclient.BaseDataClient;
import com.gct.aoid.job.properties.MutableJobConfigProperties;
import com.gct.aoid.service.AoidGkModelService;
import com.gct.aoid.service.AoidSinglegtYieldService;
import com.gct.aoid.service.AoidSinglewellConfigService;
import com.gct.aoid.service.EvaluateLiquidService;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.SGTUtil;
import com.gct.common.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * class name: EvaluateLiquidServiceImpl
 *
 * @author gxt
 * @version 1.0
 * @since 2022/9/23 上午11:31 周五
 */
@Service
public class EvaluateLiquidServiceImpl implements EvaluateLiquidService {

    @Autowired
    BaseDataClient baseDataClient;
    @Autowired
    AoidSinglegtYieldService aoidSinglegtYieldService;
    @Autowired
    AoidSinglewellConfigService aoidSinglewellConfigService;
    @Autowired
    CalPumpCard calPumpCard;
    @Autowired
    CalDiagramProduction calDiagramProduction;
    @Autowired
    AoidGkModelService aoidGkModelService;
    @Autowired
    MutableJobConfigProperties config;

    @Override
    public void evaluateLiquidWithThinOil(AoidMessage message) {
        SaveTempData saveTempData = new SaveTempData();
        BaseParam baseParam = message.getBaseParam();
        String timeValue = getTimeByWellId(baseParam.getWellId());
        String realDate = getRealDate(baseParam.getProdDate(), timeValue);
        Object calibrationObject = JSONObject.parseObject(baseDataClient.getCbPcDevWellAttrMaxUpdateTime(baseParam.getWellId(), realDate, "2"), Result.class).getData();
        if (Objects.nonNull(calibrationObject)) {
            double ratio;
            JSONObject updateTimeJson = JSONObject.parseObject(calibrationObject.toString());
            String updateDate = StringUtil.isBlankOrEmpty(updateTimeJson.getString("updateTime")) ? "" : updateTimeJson.getString("updateTime").split("\\.")[0];
            String startDate = DateTimeUtil.formatDateTime(updateDate).toLocalDate().minusDays(2) + " " + timeValue;
            String endDate = DateTimeUtil.formatDateTime(updateDate).toLocalDate().minusDays(1) + " " + timeValue;
            JSONObject ratioDataJson = JSONObject.parseObject(JSONObject.parseObject(baseDataClient.getCbPcDevWellAttrMaxUpdateTime(baseParam.getWellId(), updateDate, "3"), Result.class).getData().toString());
            if ("系统自动".equals(ratioDataJson.getString("markedBy"))) {
                DynamicParam dynamicParam = getDynamicParam(baseParam.getWellId(), startDate, endDate);
                if (Objects.nonNull(dynamicParam.getStroke()) && Objects.nonNull(dynamicParam.getFrequence())) {
                    double strokeDiff = Math.abs(baseParam.getS() - dynamicParam.getStroke());
                    double frequenceDiff = Math.abs(baseParam.getN() - dynamicParam.getFrequence());
                    if (strokeDiff > 0.5 || frequenceDiff > 0.5) {
                        Object paramObject = JSONObject.parseObject(baseDataClient.getCbPcDevWellAttrMaxUpdateTime(baseParam.getWellId(), updateDate, "2"), Result.class).getData();
                        if (Objects.nonNull(paramObject)) {
                            JSONObject paramJson = JSONObject.parseObject(paramObject.toString());
                            String newParamDate = StringUtil.isBlankOrEmpty(paramJson.getString("updateTime")) ? "" : paramJson.getString("updateTime").split("\\.")[0];
                            String startParamDate = DateTimeUtil.formatDateTime(newParamDate).toLocalDate().minusDays(1) + " " + timeValue;
                            String endParamDate = DateTimeUtil.formatDateTime(newParamDate).toLocalDate() + " " + timeValue;
                            ratio = Objects.nonNull(paramJson.getDouble("coefficient")) ? paramJson.getDouble("coefficient") : 1.0;
                            conjectureAreaRatio(message, newParamDate, startParamDate, endParamDate, ratio, timeValue);
                        }
                    }
                }
            } else {
                saveTempData.setFinalCoefficient(ratioDataJson.getDouble("coefficient"));
            }
        }
        saveLiquid(message, saveTempData);
    }

    private String getTimeByWellId(String wellId) {
        Object data = JSONObject.parseObject(baseDataClient.getOrgByWellId(wellId), Result.class).getData();
        String orgId = null;
        String timeValue = null;
        if (Objects.nonNull(data)) orgId = data.toString().split("@")[0];
        List<Map<String, String>> workAreaDate = config.getCalDailyLiqJob().getParam().getWorkAreaDate();
        for (Map<String, String> map : workAreaDate) {
            if (map.get("orgId").equals(orgId)) {
                timeValue = map.getOrDefault("time", "08:00:00");
            }
        }
        return timeValue;
    }

    private String getRealDate(LocalDateTime prodDate, String timeValue) {
        String realDate;
        String defaultTime = " 00:00:00";
        if (prodDate.toLocalTime().isBefore(DateTimeUtil.formatTime(timeValue)))
            realDate = prodDate.toLocalDate() + defaultTime;
        else realDate = prodDate.toLocalDate().plusDays(1) + defaultTime;
        return realDate;
    }

    private DynamicParam getDynamicParam(String wellId, String satrtDate, String endDate) {
        DynamicParam dynamicParam = new DynamicParam();
        double areaSum = 0.0;
        double loadDifferenceSum = 0.0;
        double strokeSum = 0.0;
        double frequenceSum = 0.0;
        double validstrokeSum = 0.0;
        double demarcateRatioSum = 0.0;
        List<AoidSinglegtYield> list = aoidSinglegtYieldService.findByJhbasecs(wellId, satrtDate, endDate);
        long size = list.size();
        if (list.isEmpty()) return dynamicParam;
        for (AoidSinglegtYield singlegt : list) {
            if (!StringUtil.isBlankOrEmpty(singlegt.getSgt())) {
                List<List<Double>> sgtList = SGTUtil.encodeToUseList(singlegt.getSgt());
                if (sgtList.size() == 1) sgtList.clear();
                if (sgtList.size() > 0) {
                    sgtList.remove(sgtList.size() - 1);
                    sgtList.add(sgtList.get(0));
                    Double[][] sgtValue = JSONObject.parseObject(sgtList.toString(), Double[][].class);
                    double areaReal = CardAreaAlgorithm.calArea(sgtValue);
                    if (areaReal > 0) areaSum += areaReal;
                }
            }
            if (Objects.nonNull(singlegt.getSxzh()) && Objects.nonNull(singlegt.getXxzh()))
                loadDifferenceSum += (singlegt.getSxzh() - singlegt.getXxzh());
            strokeSum += (Objects.nonNull(singlegt.getStroke()) ? singlegt.getStroke() : 0);
            frequenceSum += (Objects.nonNull(singlegt.getFrequence()) ? singlegt.getFrequence() : 0);
            validstrokeSum += (Objects.nonNull(singlegt.getYxcc()) ? singlegt.getYxcc() : 0);
            demarcateRatioSum += (singlegt.getBdxs() > 9 ? 1 : singlegt.getBdxs());
        }
        dynamicParam.setYxcc(validstrokeSum / size)
                .setBdxs(demarcateRatioSum / size)
                .setArea(areaSum / size)
                .setLoadDifference(loadDifferenceSum / size)
                .setStroke(strokeSum / size)
                .setFrequence(frequenceSum / size);
        return dynamicParam;
    }

    /**
     * 根据最近的一次产液量推测面积系数
     */
    private void conjectureAreaRatio(AoidMessage message, String proDateForA2, String startDate, String endDate, double ratio, String timeValue) {
        BaseParam baseParam = message.getBaseParam();
        SaveTempData saveTempData = new SaveTempData();
        if (Objects.nonNull(baseParam.getSgt())) {
            Double[][] doubles = SGTUtil.encodeToDoubleArray(baseParam.getSgt());
            double areaReal = CardAreaAlgorithm.calArea(doubles);
            DiagnoseResult result = message.getDiagnoseResult();
            if ("-1".equals(result.getResult1()) || (areaReal <= -1 || areaReal > 500)) {
                saveLiquid(message, saveTempData);
            } else {
                String proDate = DateTimeUtil.formatDateTime(proDateForA2).toLocalDate() + " 00:00:00";
                String defultTime = " " + timeValue;
                BaseDataForA2 dataForA2 = getBaseDataForA2(baseParam.getWellId(), proDate);
                //DynamicParam dynamicParam = getDynamicParam(baseParam.getWellId(), startDate, endDate);
                LocalDateTime nowDate = DateTimeUtil.formatDateTime(baseParam.getProdDate().toLocalDate() + defultTime);
                String startDateForLiq;
                String endDateForLiq;
                if (baseParam.getProdDate().isBefore(nowDate)) {
                    startDateForLiq = baseParam.getProdDate().toLocalDate().minusDays(1) + defultTime;
                    endDateForLiq = baseParam.getProdDate().toLocalDate() + defultTime;
                } else {
                    startDateForLiq = baseParam.getProdDate().toLocalDate() + defultTime;
                    endDateForLiq = baseParam.getProdDate().toLocalDate().plusDays(1) + defultTime;
                }
                List<AoidSinglegtYield> list = aoidSinglegtYieldService.findByJhbasecs(baseParam.getWellId(), startDateForLiq, endDateForLiq);
                double ratioTemp = 0.0;
                if (list.isEmpty()) ratioTemp = ratio;
                else ratioTemp = list.get(list.size() - 1).getBdxs();
                if (Objects.nonNull(dataForA2.getBj())) {
                    RealLiquid realLiquid = calRealLiquid(message, dataForA2, ratioTemp);
                    if (Objects.nonNull(realLiquid.getYxcc()) && Objects.nonNull(realLiquid.getRcl())) {
                        //判断液量是否合理
                        reasonableLiquid();
                    }
                }
                saveTempData.setParmFlag("无标定信息");
                saveLiquid(message, saveTempData);
            }
        }
    }

    private BaseDataForA2 getBaseDataForA2(String wellId, String proDate) {
        BaseDataForA2 baseDataForA2 = new BaseDataForA2();
        try {
            ProDataForCalProduction vol = JSONObject.parseObject(JSON.toJSONString(JSONObject.parseObject(baseDataClient.getA2VolDataForCalProduction(wellId, proDate), Result.class).getData()), ProDataForCalProduction.class);
            ProDataForCalProduction status = JSONObject.parseObject(JSON.toJSONString(JSONObject.parseObject(baseDataClient.getA2StatusDataForCalProduction(wellId, proDate), Result.class).getData()), ProDataForCalProduction.class);
            if (Objects.nonNull(vol) && Objects.nonNull(status)) {
                baseDataForA2.setLiqProdDaily(Objects.nonNull(vol.getLiqProdDaily()) ? vol.getLiqProdDaily() : 0)
                        .setHsl(Objects.nonNull(vol.getHsl()) ? vol.getHsl() : 0)
                        .setSmd(Objects.nonNull(vol.getSmd()) ? vol.getSmd() : 0)
                        .setYmd(Objects.nonNull(vol.getYmd()) ? vol.getYmd() : 0)
                        .setQyb(Objects.nonNull(vol.getQyb()) ? vol.getQyb() : 0)
                        .setRemarks(vol.getRemarks())
                        .setBj(Objects.nonNull(status.getBj()) ? status.getBj() : 0)
                        .setBs(Objects.nonNull(status.getBs()) ? status.getBs() : 0)
                        .setDym(Objects.nonNull(status.getDym()) ? status.getDym() : 0)
                        .setGas(Objects.nonNull(status.getGasProdDaily()) ? status.getGasProdDaily() : 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return baseDataForA2;
    }

    /**
     * 计算实时的液量值
     */
    private RealLiquid calRealLiquid(AoidMessage message, BaseDataForA2 dataForA2, double ratio) {
        BaseParam baseParam = message.getBaseParam();
        DiagnoseResult result = message.getDiagnoseResult();
        RealLiquid realLiquid = new RealLiquid();
        AoidSinglewellConfig wellConfig = aoidSinglewellConfigService.getConfigOneObjByID(1);
        Double[][] bgt;
        if (Objects.nonNull(baseParam.getSgt())) {
            Double[][] sgt = SGTUtil.encodeToDoubleArray(baseParam.getSgt());
            bgt = calPumpCard.cal_bgt(sgt, baseParam.getS(), baseParam.getIslegal(), 1.0, result.getResult2());
            if (Objects.isNull(bgt)) return realLiquid;
            CalDiagramProduction.CalProductionResult production = calDiagramProduction.calProduction(sgt, bgt, baseParam.getS(), baseParam.getN(), result.getResult2(), dataForA2.getBj(),
                    dataForA2.getBs(), dataForA2.getHsl(), dataForA2.getSmd(), dataForA2.getYmd(), dataForA2.getQyb(), dataForA2.getDym(), ratio, wellConfig);
            realLiquid.setRcl(production.rcyl1).setYxcc(production.yxcc);
        }
        return realLiquid;
    }

    /**
     * 评估产液量是否合理
     */
    private void reasonableLiquid() {

    }

    private void saveLiquid(AoidMessage message, SaveTempData tempData) {
        BaseParam baseParam = message.getBaseParam();
        AoidSinglegtYield singlegtYield = new AoidSinglegtYield();
        ProDataForCalProduction a2DataStatus = null;
        ProDataForCalProduction a2DataVol = null;
        ProDataForCalProduction wellMeterteam = null;
        try {
            if (Objects.nonNull(baseParam) && Objects.nonNull(baseParam.getWellId()) && Objects.nonNull(baseParam.getProdDate())) {
                singlegtYield.setWellId(baseParam.getWellId()).setWellCommonName(baseParam.getWellname()).setProdDate(baseParam.getProdDate())
                        .setStroke(baseParam.getS()).setFrequence(baseParam.getN()).setSxzh(baseParam.getLMax()).setXxzh(baseParam.getLMin())
                        .setSgt(baseParam.getSgt()).setSgtIsLegal(baseParam.getIslegal());
                String a2Date = DateTimeUtil.formatString(baseParam.getProdDate().toLocalDate());
                a2DataStatus = (ProDataForCalProduction) JSONObject.parseObject(baseDataClient.getA2StatusDataForCalProduction(baseParam.getWellId(), a2Date), Result.class).toFiguredOutObject(ProDataForCalProduction.class);
                a2DataVol = (ProDataForCalProduction) JSONObject.parseObject(baseDataClient.getA2VolDataForCalProduction(baseParam.getWellId(), a2Date), Result.class).toFiguredOutObject(ProDataForCalProduction.class);
                String tempDate = DateTimeUtil.formatString(baseParam.getProdDate().toLocalDate());
                wellMeterteam = JSONObject.parseObject(JSON.toJSONString(JSONObject.parseObject(baseDataClient.getWellMeterTeamForCalProduction(baseParam.getWellId(), tempDate), Result.class).getData()), ProDataForCalProduction.class);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        DiagnoseResult diagnoseResult = message.getDiagnoseResult();
        AoidGkModel model = null;
        Double[][] bgt = null;
        Double[][] sgt = null;
        Double realArea = null;
        if (Objects.nonNull(diagnoseResult)) {
            singlegtYield.setEczdjg(diagnoseResult.getResult2()).setZdjg(diagnoseResult.getResult1());
            if (Integer.parseInt(diagnoseResult.getResult1()) > 1) singlegtYield.setSgtIsLegal(1);
            else singlegtYield.setSgtIsLegal(0);
            model = aoidGkModelService.getGKModelByResultType(diagnoseResult.getResult2());
            if (Objects.nonNull(baseParam.getSgt())) {
                sgt = SGTUtil.encodeToDoubleArray(baseParam.getSgt());
                realArea = CardAreaAlgorithm.calArea(sgt);
                bgt = calPumpCard.cal_bgt(sgt, baseParam.getS(), baseParam.getIslegal(), 1.0, diagnoseResult.getResult2());
            }
        }
        double finalCoefficient = tempData.getFinalCoefficient();
        if (finalCoefficient > 9) finalCoefficient = 1;
        if (Objects.nonNull(model) && !Objects.equals("0", model.getTriggerFlag())) {
            finalCoefficient = finalCoefficient * model.getGkxs();
            finalCoefficient = finalCoefficient < 0.1 ? 0.15 : finalCoefficient;
        }
        CalDiagramProduction.CalProductionResult productionResult = new CalDiagramProduction.CalProductionResult();
        AoidSinglewellConfig aoidSinglewellConfig = aoidSinglewellConfigService.getConfigOneObjByID(1);
        boolean check = checkInputDataBeforeCalDiagramProduction(productionResult, a2DataStatus, a2DataVol, bgt, message, model, aoidSinglewellConfig, realArea);
        singlegtYield.setCalIsNormal(check ? 1 : 0);
        if (check) {
            assert a2DataStatus != null;
            assert a2DataVol != null;
            productionResult = calDiagramProduction.calProduction(sgt, bgt, baseParam.getS(), baseParam.getN(), diagnoseResult.getResult2(),
                    a2DataStatus.getBj(), a2DataStatus.getBj(), a2DataVol.getHsl(), a2DataVol.getSmd(), a2DataVol.getYmd(), a2DataVol.getQyb(),
                    a2DataStatus.getDym(), finalCoefficient, aoidSinglewellConfig);
        }
        saveData(singlegtYield, productionResult, a2DataStatus, a2DataVol, wellMeterteam, bgt, finalCoefficient, tempData);
    }

    private boolean checkInputDataBeforeCalDiagramProduction(CalDiagramProduction.CalProductionResult result, ProDataForCalProduction status, ProDataForCalProduction vol,
                                                             Double[][] bgt, AoidMessage message, AoidGkModel model, AoidSinglewellConfig aoidSinglewellConfig,
                                                             double realArea) {
        result.remark = "";
        BaseParam baseParam = message.getBaseParam();
        DiagnoseResult diagnoseResult = message.getDiagnoseResult();
        if (Objects.isNull(bgt) || bgt.length == 0) result.appendRemark("泵功图无法计算");
        if (realArea <= -1 || realArea > 500) result.appendRemark("功图错误");
        if (Objects.isNull(message.getBaseParam())) result.appendRemark("缺失功图参数");
        else {
            if (baseParam.getIslegal() != 1) {
                if (Objects.nonNull(diagnoseResult) && !StringUtil.isBlankOrEmpty(diagnoseResult.getResult2()))
                    result.appendRemark("功图不合法:" + diagnoseResult.getResult2());
                else result.appendRemark("功图不合法");
            }
            if (baseParam.getN() == 0) result.appendRemark("冲次为0");
            if (baseParam.getS() == 0) result.appendRemark("冲程为0");
        }
        if (Objects.isNull(status)) result.appendRemark("缺失井状态日数据");
        else {
            if (Objects.isNull(status.getBj())) result.appendRemark("缺失泵径");
            if (Objects.isNull(status.getBs())) result.appendRemark("缺失泵深");
            if (Objects.isNull(status.getDym())) result.appendRemark("缺失动液面");
        }
        if (Objects.isNull(vol)) result.appendRemark("缺失井产量日数据");
        else {
            if (Objects.isNull(vol.getHsl())) result.appendRemark("缺失含水率");
            if (Objects.isNull(vol.getSmd())) result.appendRemark("缺失水比重");
            if (Objects.isNull(vol.getQyb())) result.appendRemark("缺失气油比");
            if (Objects.isNull(vol.getYmd())) result.appendRemark("缺失油比重");
        }
        if (Objects.isNull(model))
            result.appendRemark("缺失工况对应计量模型");
        if (Objects.isNull(aoidSinglewellConfig)) result.appendRemark("缺失有效冲程计算模型");
        if (Objects.isNull(diagnoseResult) || Objects.isNull(diagnoseResult.getResult1())
                || Objects.isNull(diagnoseResult.getResult2()) || diagnoseResult.getResult1().isEmpty()
                || diagnoseResult.getResult2().isEmpty()) result.appendRemark("缺失诊断结果数据");
        return result.remark.isEmpty();
    }

    private void saveData(AoidSinglegtYield aoidSinglegtYield, CalDiagramProduction.CalProductionResult production, ProDataForCalProduction a2DataStatus,
                          ProDataForCalProduction a2DataVol, ProDataForCalProduction wellMeterteam, Double[][] bgt, double finalCoefficient,
                          SaveTempData saveTempData) {
        double Tlvd = 0;
        String defultProdDate = "1970-01-01 00:00:00";
        if (a2DataStatus != null) {
            aoidSinglegtYield.setBj(a2DataStatus.getBj());
            aoidSinglegtYield.setBs(a2DataStatus.getBs());
            if (Objects.nonNull(a2DataStatus.getBs()) && Objects.nonNull(a2DataStatus.getDym()))
                aoidSinglegtYield.setCmd(Math.round((a2DataStatus.getBs() - a2DataStatus.getDym()) * 10) / 10d);
            aoidSinglegtYield.setYzzj(a2DataStatus.getBj());
            aoidSinglegtYield.setHy(a2DataStatus.getBackPres());
            aoidSinglegtYield.setYy(a2DataStatus.getTubingPres());
            aoidSinglegtYield.setTy(a2DataStatus.getCasingPres());
            aoidSinglegtYield.setA2Dym(a2DataStatus.getDym());
        }
        if (a2DataVol != null) {
            aoidSinglegtYield.setHsl(a2DataVol.getHsl());
            aoidSinglegtYield.setSmd(a2DataVol.getSmd());
            aoidSinglegtYield.setYmd(a2DataVol.getYmd());
            aoidSinglegtYield.setQyb(a2DataVol.getQyb());
            aoidSinglegtYield.setA2LiqProdDaily(a2DataVol.getLiqProdDaily());
            aoidSinglegtYield.setA2OilProdDaily(a2DataVol.getOilProdDaily());
            aoidSinglegtYield.setA2WaterProdDaily(a2DataVol.getWaterProdDaily());
            aoidSinglegtYield.setA2GasProdDaily(a2DataVol.getGasProdDaily());
            aoidSinglegtYield.setA2WaterCut(a2DataVol.getHsl());
            aoidSinglegtYield.setA2Remark(a2DataVol.getRemarks());
            aoidSinglegtYield.setA2ProdTime(a2DataVol.getProdTime());
        }
        if (wellMeterteam != null) aoidSinglegtYield.setTestLiquVolDaily(wellMeterteam.getTlvd());
        else aoidSinglegtYield.setTestLiquVolDaily(Tlvd);
        if (bgt != null && bgt.length > 0) {
            aoidSinglegtYield.setBgt(SGTUtil.encodeToString(bgt));
        }
        if (production != null) {
            aoidSinglegtYield.setRemark(production.remark);
            aoidSinglegtYield.setOilProdDaily(production.rcyl);
            aoidSinglegtYield.setCreateTime(LocalDateTime.now());//创建时间
            aoidSinglegtYield.setLiqProdDaily(production.rcyl1);
            aoidSinglegtYield.setGasProdDaily(production.rcql);
            aoidSinglegtYield.setBx(production.bx);
            aoidSinglegtYield.setDym(production.dym);//新增动液面计算
            aoidSinglegtYield.setLlpl(production.llpl);
            aoidSinglegtYield.setZcc(production.zcc);
            aoidSinglegtYield.setSjcc(production.sjcc);
            aoidSinglegtYield.setYxcc(production.yxcc);
            aoidSinglegtYield.setCcss(production.ccss);
            aoidSinglegtYield.setLsss(production.lsss);
            aoidSinglegtYield.setGyss(production.gyss);
            aoidSinglegtYield.setWaterProdDaily(production.rcsl);
            aoidSinglegtYield.setBdxs(finalCoefficient);//标定系数
            aoidSinglegtYield.setStrokeRatio(production.l_StrokeRatio);
        }
        aoidSinglegtYield.setScsj(24d);
        aoidSinglegtYield.setLoadDifference(saveTempData.getLoadDifference());
        aoidSinglegtYield.setArea(String.valueOf(saveTempData.getArea()).length() > 7 ?
                String.valueOf(saveTempData.getArea()).substring(0, 5) : String.valueOf(saveTempData.getArea()));
        if (Objects.nonNull(saveTempData.getProdDateForA2()))
            aoidSinglegtYield.setComparisonProdDate(saveTempData.getProdDateForA2());
        else aoidSinglegtYield.setComparisonProdDate(DateTimeUtil.formatDateTime(defultProdDate));
        aoidSinglegtYield.setComparisonCoefficient(saveTempData.getRatio());
        aoidSinglegtYield.setComparisonYield(saveTempData.getLiqProdDailyForA2());
        aoidSinglegtYield.setComparisonYxcc(String.valueOf(saveTempData.getValidStroke()).length() > 7 ?
                String.valueOf(saveTempData.getValidStroke()).substring(0, 5) : String.valueOf(saveTempData.getValidStroke()));
        aoidSinglegtYield.setComparisonLoadDifference(String.valueOf(saveTempData.getLoadDifferenceForAvg()).length() > 7 ?
                String.valueOf(saveTempData.getLoadDifferenceForAvg()).substring(0, 5) : String.valueOf(saveTempData.getLoadDifferenceForAvg()));
        aoidSinglegtYield.setComparisonArea(String.valueOf(saveTempData.getAreaForAvg()).length() > 7 ?
                String.valueOf(saveTempData.getAreaForAvg()).substring(0, 5) : String.valueOf(saveTempData.getAreaForAvg()));
        aoidSinglegtYield.setYxccRate(String.valueOf(saveTempData.getYxccRate()).length() > 7 ?
                String.valueOf(saveTempData.getYxccRate()).substring(0, 5) : String.valueOf(saveTempData.getYxccRate()));
        aoidSinglegtYield.setLoadDifferenceRate(String.valueOf(saveTempData.getLoadDifferenceRate()).length() > 7 ?
                String.valueOf(saveTempData.getLoadDifferenceRate()).substring(0, 5) : String.valueOf(saveTempData.getLoadDifferenceRate()));
        aoidSinglegtYield.setAreaRate(String.valueOf(saveTempData.getAreaRate()).length() > 7 ?
                String.valueOf(saveTempData.getAreaRate()).substring(0, 5) : String.valueOf(saveTempData.getAreaRate()));
        aoidSinglegtYield.setMeasureFlag(saveTempData.getMeasures());
        aoidSinglegtYield.setIsStopWell(saveTempData.getIsStop());
        aoidSinglegtYield.setParmExplain(saveTempData.getParmExplain() + "--新算法");
        aoidSinglegtYield.setParmFlag(saveTempData.getParmFlag());
        if (!aoidSinglegtYieldService.update(aoidSinglegtYield, new LambdaUpdateWrapper<AoidSinglegtYield>()
                .eq(AoidSinglegtYield::getWellId, aoidSinglegtYield.getWellId())
                .eq(AoidSinglegtYield::getProdDate, aoidSinglegtYield.getProdDate())))
            aoidSinglegtYieldService.save(aoidSinglegtYield);
    }

    @Override
    public void evaluateLiquidWithHeavyOil(AoidMessage message) {

    }
}
